"""
任务界面模块 - TUI模式下的任务进度和日志显示
"""

import re
import time
import threading
import collections

from rich.console import Console, Group
from rich.panel import Panel
from rich.text import Text
from rich.layout import Layout
from rich.progress import Progress, BarColumn, TextColumn, TimeRemainingColumn, TimeElapsedColumn, SpinnerColumn


class TaskUI:
    """TUI任务界面 - 显示进度条、日志和实时对照"""

    def __init__(self, parent_cli=None, i18n=None):
        self._lock = threading.RLock()
        self.parent_cli = parent_cli
        self.i18n = i18n

        # 根据配置决定日志保留数量
        self.show_detailed = parent_cli.config.get("show_detailed_logs", False) if parent_cli else False
        self.logs = collections.deque(maxlen=100)  # 统一保留100条日志，方便回溯

        self.log_filter = "ALL"
        self.taken_over = False
        self.web_task_manager = None
        self.last_error = ""
        self.log_file = None  # 实时的日志文件句柄

        # 实时对照内容存储 (仅在详细模式使用)
        self.current_source = Text("Waiting...", style="dim")
        self.current_translation = Text("Waiting...", style="dim")

        self.progress = Progress(
            SpinnerColumn(),
            TextColumn("[bold blue]{task.fields[action]}", justify="left"),
            BarColumn(bar_width=None),
            "[progress.percentage]{task.percentage:>3.0f}%",
            TimeElapsedColumn(),
            TextColumn("/"),
            TimeRemainingColumn(),
            expand=True
        )
        self.task_id = self.progress.add_task("", total=100, action=self._get_i18n('label_initializing'))

        # 初始化布局
        self.layout = Layout()
        if self.show_detailed:
            # 详细模式：三段式 (Header + Body + Footer)
            self.layout.split(
                Layout(name="header", size=3),
                Layout(name="body", ratio=1),
                Layout(name="footer", size=15)
            )
            self.layout["body"].split_row(
                Layout(name="source_pane", ratio=1),
                Layout(name="target_pane", ratio=1)
            )
            self.layout["footer"].split(
                Layout(name="small_logs", ratio=1),
                Layout(name="stats", size=5)
            )
        else:
            # 经典模式：上下两段式
            self.layout.split(
                Layout(name="upper", ratio=4, minimum_size=10),
                Layout(name="lower", size=6)
            )

        self.stats_text = Text("Initializing stats...", style="cyan")
        self.current_status_key = 'label_status_normal'
        self.current_status_color = 'green'
        self.current_border_color = "green"

        self.refresh_layout()

    def _get_i18n(self, key):
        """获取国际化文本"""
        if self.i18n:
            return self.i18n.get(key)
        return key

    def refresh_layout(self):
        """刷新 TUI 渲染内容"""
        with self._lock:
            if self.show_detailed:
                # 统计行数
                s_lines = len(self.current_source.plain.split('\n')) if self.current_source.plain else 0
                t_lines = len(self.current_translation.plain.split('\n')) if self.current_translation.plain else 0

                # 渲染详细对照模式
                self.layout["source_pane"].update(Panel(
                    self.current_source,
                    title=f"[bold magenta]SOURCE ({s_lines} lines)[/]",
                    border_style="magenta",
                    padding=(0, 1)
                ))
                self.layout["target_pane"].update(Panel(
                    self.current_translation,
                    title=f"[bold green]TRANSLATION ({t_lines} lines)[/]",
                    border_style="green",
                    padding=(0, 1)
                ))
                # 底部小日志窗格
                log_group = Group(*list(self.logs)[-8:])
                self.layout["small_logs"].update(Panel(log_group, title="System Logs", border_style="blue"))
                self.panel_group = Group(self.progress, self.stats_text)
                self.layout["stats"].update(Panel(self.panel_group, title="Progress & Metrics", border_style=self.current_border_color))
            else:
                # 渲染经典滚动模式
                log_group = Group(*list(self.logs)[-35:])
                self.layout["upper"].update(Panel(log_group, title=f"Logs ({self.log_filter})", border_style="blue", padding=(0, 1)))
                self.panel_group = Group(self.progress, self.stats_text)
                self.layout["lower"].update(Panel(self.panel_group, title="Progress & Stats", border_style=self.current_border_color))

    def update_status(self, event, data):
        with self._lock:
            status = data.get("status", "normal") if isinstance(data, dict) else "normal"
            color_map = {"normal": "green", "fixing": "yellow", "warning": "yellow", "error": "red", "paused": "yellow", "critical_error": "red"}
            status_key_map = {
                "normal": "label_status_normal",
                "fixing": "label_status_fixing",
                "warning": "label_status_warning",
                "error": "label_status_error",
                "paused": "label_status_paused",
                "critical_error": "label_status_critical_error"
            }

            self.current_status_key = status_key_map.get(status, "label_status_normal")
            self.current_status_color = color_map.get(status, "green")
            self.current_border_color = self.current_status_color
            self.update_progress(None, {})

    def _is_error_log(self, log_item: Text):
        """Heuristically determines if a log entry is an error."""
        text = log_item.plain.lower()
        err_words = ['error', 'fail', 'failed', 'exception', 'traceback', 'critical', 'panic', '✗']

        has_red_style = False
        for span in log_item.spans:
            s = span.style
            if isinstance(s, str):
                if "red" in s: has_red_style = True; break
            elif hasattr(s, "color") and s.color and (s.color.name == "red" or s.color.number == 1):
                has_red_style = True; break

        return has_red_style or any(word in text for word in err_words)

    def refresh_logs(self):
        """Renders the log panel according to the current filter."""
        with self._lock:
            if self.log_filter == "ALL":
                display_logs = list(self.logs)[-35:]
            else:  # ERROR
                display_logs = [log for log in self.logs if self._is_error_log(log)][-35:]

            log_group = Group(*display_logs)
            self.layout["upper"].update(Panel(log_group, title=f"Logs ({self.log_filter})", border_style="blue", padding=(0, 1)))

    def toggle_log_filter(self):
        self.log_filter = "ERROR" if self.log_filter == "ALL" else "ALL"
        self.log(f"[dim]Log view set to: {self.log_filter}[/dim]")
        self.refresh_logs()

    def on_source_data(self, event, data):
        """接收原文数据的事件回调"""
        if not self.show_detailed: return
        pass

    def on_result_data(self, event, data):
        """接收译文数据的事件回调"""
        if not self.show_detailed: return
        if not isinstance(data, dict): return
        raw_content = str(data.get("data", ""))
        source_content = data.get("source")
        if not raw_content and not source_content: return

        with self._lock:
            if source_content:
                clean_source = "".join([c for c in str(source_content) if c == '\n' or c >= ' '])
                self.current_source = Text(clean_source, style="magenta")

            if raw_content:
                clean_content = "".join([c for c in raw_content if c == '\n' or c >= ' '])
                self.current_translation = Text(clean_content, style="green")

            if self.web_task_manager:
                self.web_task_manager.push_comparison(
                    str(self.current_source.plain),
                    str(self.current_translation.plain)
                )

            self._last_result_time = time.time()
            self.refresh_layout()

    def log(self, msg):
        # 1. 预处理：将对象转为字符串
        if not isinstance(msg, str):
            from io import StringIO
            with StringIO() as buf:
                temp_console = Console(file=buf, force_terminal=True, width=120)
                temp_console.print(msg)
                msg_str = buf.getvalue()
        else:
            msg_str = msg

        # 2. 拦截实时对照信号 (双通道补丁)
        if "<<<RAW_RESULT>>>" in msg_str:
            if time.time() - getattr(self, "_last_result_time", 0) < 0.5:
                return

            try:
                data = msg_str.split("<<<RAW_RESULT>>>")[1].strip()
                if data:
                    with self._lock:
                        clean = "".join([c for c in data if c == '\n' or c >= ' '])
                        self.current_translation = Text(clean, style="green")
                        if self.web_task_manager:
                            self.web_task_manager.push_comparison(
                                str(self.current_source.plain),
                                str(self.current_translation.plain)
                            )
                        self.refresh_layout()
            except: pass
            return

        # 3. 过滤私有标签和状态
        if "<<<" in msg_str and ">>>" in msg_str: return
        if "[STATUS]" in msg_str: return

        clean_msg = msg_str.strip()
        if not clean_msg: return

        # Push to WebServer
        if self.web_task_manager:
            plain_msg = re.sub(r'\[/?[a-zA-Z\s]+\]', '', clean_msg)
            self.web_task_manager.push_log(plain_msg)

        current_time = time.time()
        if hasattr(self, "_last_msg") and self._last_msg == clean_msg and (current_time - getattr(self, "_last_msg_time", 0)) < 0.3:
            return
        self._last_msg, self._last_msg_time = clean_msg, current_time

        # Real-time File Logging
        timestamp = f"[{time.strftime('%H:%M:%S')}] "
        if self.log_file:
            try:
                plain_log = re.sub(r'\[/?[a-zA-Z\s]+\]', '', clean_msg)
                self.log_file.write(timestamp + plain_log + "\n")
                self.log_file.flush()
            except: pass

        if self.taken_over: return

        # 4. 构造日志内容并刷新
        try:
            new_log = Text.from_markup(timestamp + clean_msg)
        except:
            new_log = Text(timestamp + clean_msg)

        with self._lock:
            self.logs.append(new_log)

            # 自动错误监测补丁
            if self._is_error_log(new_log) and self.parent_cli:
                lower_msg = clean_msg.lower()
                if any(w in lower_msg for w in ['traceback', 'panic', 'exception', 'fatal']):
                    if self.current_status_color != 'red':
                        self.current_status_color = 'red'
                        self.current_border_color = 'red'

                if "traceback" in lower_msg or "panic" in lower_msg:
                    self.parent_cli._is_critical_failure = True
                    if not getattr(self.parent_cli, "_last_crash_msg", None):
                        self.parent_cli._last_crash_msg = clean_msg

                api_error_keywords = ['401', '403', '429', '500', '502', '503', 'timeout', 'connection', 'ssl', 'rate_limit']
                if any(k in lower_msg for k in api_error_keywords):
                    self.parent_cli._api_error_count += 1
                    if len(self.parent_cli._api_error_messages) < 10:
                        self.parent_cli._api_error_messages.append(clean_msg)
                    if self.parent_cli._api_error_count >= 3 and not self.parent_cli._show_diagnostic_hint:
                        self.parent_cli._show_diagnostic_hint = True

            self.refresh_layout()

    def update_progress(self, event, data):
        with self._lock:
            if not hasattr(self, "_last_progress_data"):
                self._last_progress_data = {"line": 0, "total_line": 1, "token": 0, "time": 0, "file_name": "...", "total_requests": 0, "error_requests": 0}

            if data and isinstance(data, dict):
                self._last_progress_data.update(data)
            d = self._last_progress_data
            completed, total = d["line"], d["total_line"]
            tokens, elapsed = d["token"], d["time"]

            calc_tokens = d.get("session_token", tokens)
            calc_requests = d.get("session_requests", d.get("total_requests", 0))

            if elapsed > 0:
                rpm = (calc_requests / (elapsed / 60))
                tpm_k = (calc_tokens / (elapsed / 60) / 1000)
            else: rpm, tpm_k = 0, 0

            total_req = d.get("total_requests", 0)
            success_req = d.get("success_requests", 0)
            error_req = d.get("error_requests", 0)

            if total_req > 0:
                s_rate = (success_req / total_req) * 100
                e_rate = (error_req / total_req) * 100
            else:
                s_rate, e_rate = 0, 0

            # 更新 Header (详细模式专用)
            if self.show_detailed and self.parent_cli:
                cfg = self.parent_cli.config
                src = cfg.get("source_language", "Unknown")
                tgt = cfg.get("target_language", "Unknown")
                tp = cfg.get("target_platform", "Unknown")
                status_line = f"[bold cyan]AITF[/bold cyan] | {src} -> {tgt} | API: {tp} | Progress: {completed}/{total}"
                self.layout["header"].update(Panel(status_line, title="Status", border_style="cyan"))

            if self.taken_over:
                target_pane = "body" if self.show_detailed else "upper"

            # 检查是否为队列模式
            is_queue_mode = False
            if self.parent_cli and hasattr(self.parent_cli, '_is_queue_mode'):
                is_queue_mode = self.parent_cli._is_queue_mode

            current_file = d.get("file_name", "...")

            if is_queue_mode and self.parent_cli:
                try:
                    import os
                    from ModuleFolders.Service.TaskQueue.QueueManager import QueueManager
                    qm = QueueManager()
                    if qm.current_task_index >= 0 and qm.current_task_index < len(qm.tasks):
                        current_task = qm.tasks[qm.current_task_index]
                        if current_task and hasattr(current_task, 'input_path'):
                            current_file = os.path.basename(current_task.input_path)
                except:
                    pass

            if self.web_task_manager:
                self.web_task_manager.push_stats({
                    "rpm": rpm,
                    "tpm": tpm_k,
                    "totalProgress": total,
                    "completedProgress": completed,
                    "totalTokens": tokens,
                    "currentFile": current_file,
                    "status": "running",
                    "successRate": s_rate,
                    "errorRate": e_rate
                })

            rpm_str = f"{rpm:.2f}"
            tpm_str = f"{tpm_k:.2f}k"
            status_text = self._get_i18n(self.current_status_key)

            if is_queue_mode:
                hotkeys = self._get_i18n("label_shortcuts_queue")
            else:
                hotkeys = self._get_i18n("label_shortcuts")

            diagnostic_hint = ""
            if self.parent_cli and getattr(self.parent_cli, '_show_diagnostic_hint', False):
                diagnostic_hint = f"\n[bold yellow]{self._get_i18n('msg_api_error_hint')}[/bold yellow]"

            current_threads = "Auto"
            if self.parent_cli and hasattr(self.parent_cli, 'task_executor'):
                current_threads = self.parent_cli.task_executor.config.actual_thread_counts

            stats_markup = (
                f"File: [bold]{current_file}[/] | Progress: [bold]{completed}/{total}[/] | Threads: [bold]{current_threads}[/] | RPM: [bold]{rpm_str}[/] | TPM: [bold]{tpm_str}[/]\n"
                f"S-Rate: [bold green]{s_rate:.1f}%[/] | E-Rate: [bold red]{e_rate:.1f}%[/] | Tokens: [bold]{tokens}[/] | Status: [{self.current_status_color}]{status_text}[/{self.current_status_color}] | {hotkeys}{diagnostic_hint}"
            )
            self.stats_text = Text.from_markup(stats_markup, style="cyan")

            is_start = data.get('is_start') if isinstance(data, dict) else False
            if is_start:
                self.progress.reset(self.task_id, total=total, completed=completed, action=self._get_i18n('label_processing'))
            else:
                self.progress.update(self.task_id, total=total, completed=completed, action=self._get_i18n('label_processing'))

            self.refresh_layout()



from pathlib import Path

from ModuleFolders.Domain.FileConverter.BaseConverter import BaseFileConverter


class OfficeFileConverter(BaseFileConverter):

    FILE_SUFFIX_MAPPING = {
        ".doc": 0,  # wdFormatDocument
        ".rtf": 6,  # wdFormatRTF
        ".docx": 16,  # wdFormatDocumentDefault
        ".pdf": 17,  # wdFormatPDF
    }

    def __enter__(self):
        try:
            import pythoncom
            from win32com import client
        except ImportError:
            raise RuntimeError("提示：Office文件转换功能仅在Windows系统上可用（需要Microsoft Office和pywin32）") from None

        pythoncom.CoInitialize()
        try:
            self.office = client.Dispatch("Word.Application")
            self.office.Visible = False  # 不显示 Word 界面
            self.office.DisplayAlerts = False  # 关闭所有弹窗
            self.office.AutomationSecurity = 1  # 禁用宏和安全性弹窗（重要！）
            return self
        except Exception as e:
            pythoncom.CoUninitialize()
            raise RuntimeError("不能打开Word程序，请确保安装了 Microsoft Office") from e

    def __exit__(self, exc_type, exc, exc_tb):
        try:
            import pythoncom
        except ImportError:
            return

        # Dispatch函数会复用已有的Word进程，多次退出会导致后面的com对象没有Quit函数
        if self.office and hasattr(self.office, 'Quit'):
            self.office.Quit()
        pythoncom.CoUninitialize()

    def can_convert(self, input_file_path: Path, output_file_path: Path) -> bool:
        # 输出文件类型不在类型映射中暂不支持
        if output_file_path.suffix not in self.FILE_SUFFIX_MAPPING:
            return False

        # wps 不支持 pdf 转其他格式
        if input_file_path.suffix != '.pdf':
            return True
        app_name = self.office.Name.lower()
        is_wps = 'wps' in app_name or 'kingsoft' in app_name
        if is_wps:
            print("wps不支持把pdf文件转换为其他格式")
        return not is_wps

    def convert_file(self, input_file_path: Path, output_file_path: Path):
        if not output_file_path.parent.exists():
            output_file_path.parent.mkdir(parents=True)
        # 打开文件另存为
        doc = self.office.Documents.Open(str(input_file_path), ReadOnly=1)
        try:
            # 禁用校对，防止把段落切碎
            doc_range = doc.Content
            doc_range.NoProofing = True

            doc.SaveAs(str(output_file_path), self.FILE_SUFFIX_MAPPING[output_file_path.suffix])
        finally:
            doc.Close()



"""
TUI Editor工具函数模块
包含各种辅助功能和工具方法
"""
import os
import time
import json
from typing import Dict, List, Optional, Tuple, Any


class EditorUtils:
    """编辑器工具类"""

    @staticmethod
    def calculate_text_metrics(text: str) -> Dict:
        """计算文本指标"""
        if not text:
            return {
                'chars': 0,
                'chars_no_space': 0,
                'words': 0,
                'lines': 0,
                'paragraphs': 0
            }

        lines = text.split('\n')
        paragraphs = [p.strip() for p in text.split('\n\n') if p.strip()]
        words = text.split()

        return {
            'chars': len(text),
            'chars_no_space': len(text.replace(' ', '')),
            'words': len(words),
            'lines': len(lines),
            'paragraphs': len(paragraphs)
        }

    @staticmethod
    def format_file_size(size_bytes: int) -> str:
        """格式化文件大小显示"""
        if size_bytes < 1024:
            return f"{size_bytes} B"
        elif size_bytes < 1024 * 1024:
            return f"{size_bytes / 1024:.1f} KB"
        elif size_bytes < 1024 * 1024 * 1024:
            return f"{size_bytes / (1024 * 1024):.1f} MB"
        else:
            return f"{size_bytes / (1024 * 1024 * 1024):.1f} GB"

    @staticmethod
    def truncate_text(text: str, max_length: int, suffix: str = "...") -> str:
        """截断文本并添加省略号"""
        if len(text) <= max_length:
            return text
        return text[:max_length - len(suffix)] + suffix

    @staticmethod
    def find_text_differences(original: str, modified: str) -> List[Dict]:
        """查找两个文本之间的差异"""
        differences = []

        # 简单的行级差异比较
        original_lines = original.split('\n')
        modified_lines = modified.split('\n')

        max_lines = max(len(original_lines), len(modified_lines))

        for i in range(max_lines):
            original_line = original_lines[i] if i < len(original_lines) else ""
            modified_line = modified_lines[i] if i < len(modified_lines) else ""

            if original_line != modified_line:
                differences.append({
                    'line_number': i + 1,
                    'type': 'modified' if original_line and modified_line else
                            'added' if not original_line else 'deleted',
                    'original': original_line,
                    'modified': modified_line
                })

        return differences

    @staticmethod
    def search_text(text: str, query: str, case_sensitive: bool = False) -> List[Dict]:
        """在文本中搜索指定内容"""
        if not query:
            return []

        search_text = text if case_sensitive else text.lower()
        search_query = query if case_sensitive else query.lower()

        results = []
        lines = text.split('\n')

        for line_num, line in enumerate(lines):
            search_line = line if case_sensitive else line.lower()
            start = 0

            while True:
                pos = search_line.find(search_query, start)
                if pos == -1:
                    break

                results.append({
                    'line': line_num + 1,
                    'column': pos + 1,
                    'text': line,
                    'match_start': pos,
                    'match_end': pos + len(query),
                    'context': EditorUtils.get_text_context(lines, line_num, pos, len(query))
                })

                start = pos + 1

        return results

    @staticmethod
    def get_text_context(lines: List[str], line_num: int, pos: int, match_length: int, context_size: int = 2) -> Dict:
        """获取匹配文本的上下文"""
        context = {
            'before_lines': [],
            'current_line': lines[line_num] if line_num < len(lines) else "",
            'after_lines': [],
            'match_highlight': (pos, pos + match_length)
        }

        # 获取前面的行
        for i in range(max(0, line_num - context_size), line_num):
            context['before_lines'].append(lines[i])

        # 获取后面的行
        for i in range(line_num + 1, min(len(lines), line_num + context_size + 1)):
            context['after_lines'].append(lines[i])

        return context

    @staticmethod
    def create_backup_filename(original_path: str) -> str:
        """创建备份文件名"""
        timestamp = time.strftime("%Y%m%d_%H%M%S")
        base_path, ext = os.path.splitext(original_path)
        return f"{base_path}_backup_{timestamp}{ext}"

    @staticmethod
    def validate_json_data(data: Any) -> Tuple[bool, str]:
        """验证JSON数据的有效性"""
        try:
            if isinstance(data, str):
                json.loads(data)
            else:
                json.dumps(data)
            return True, "Valid JSON"
        except json.JSONDecodeError as e:
            return False, f"Invalid JSON: {e}"
        except Exception as e:
            return False, f"Error: {e}"

    @staticmethod
    def split_text_by_sentences(text: str) -> List[str]:
        """按句子分割文本"""
        import re
        # 简单的句子分割规则
        sentence_endings = r'[.!?。！？]'
        sentences = re.split(sentence_endings, text)

        # 清理并过滤空句子
        clean_sentences = []
        for sentence in sentences:
            sentence = sentence.strip()
            if sentence:
                clean_sentences.append(sentence)

        return clean_sentences

    @staticmethod
    def calculate_edit_distance(str1: str, str2: str) -> int:
        """计算两个字符串的编辑距离"""
        if not str1:
            return len(str2)
        if not str2:
            return len(str1)

        # 动态规划计算Levenshtein距离
        m, n = len(str1), len(str2)
        dp = [[0] * (n + 1) for _ in range(m + 1)]

        for i in range(m + 1):
            dp[i][0] = i
        for j in range(n + 1):
            dp[0][j] = j

        for i in range(1, m + 1):
            for j in range(1, n + 1):
                if str1[i - 1] == str2[j - 1]:
                    dp[i][j] = dp[i - 1][j - 1]
                else:
                    dp[i][j] = min(
                        dp[i - 1][j] + 1,    # 删除
                        dp[i][j - 1] + 1,    # 插入
                        dp[i - 1][j - 1] + 1 # 替换
                    )

        return dp[m][n]

    @staticmethod
    def extract_numbers_from_text(text: str) -> List[Dict]:
        """从文本中提取数字"""
        import re
        number_pattern = r'-?\d+(?:\.\d+)?'
        matches = []

        for match in re.finditer(number_pattern, text):
            try:
                value = float(match.group()) if '.' in match.group() else int(match.group())
                matches.append({
                    'value': value,
                    'start': match.start(),
                    'end': match.end(),
                    'text': match.group()
                })
            except ValueError:
                continue

        return matches

    @staticmethod
    def sanitize_filename(filename: str) -> str:
        """清理文件名，移除不合法字符"""
        import re
        # 移除Windows文件名中的非法字符
        illegal_chars = r'[<>:"/\\|?*]'
        sanitized = re.sub(illegal_chars, '_', filename)

        # 移除开头和结尾的空格和点
        sanitized = sanitized.strip(' .')

        # 确保不是Windows保留名称
        reserved_names = [
            'CON', 'PRN', 'AUX', 'NUL',
            'COM1', 'COM2', 'COM3', 'COM4', 'COM5', 'COM6', 'COM7', 'COM8', 'COM9',
            'LPT1', 'LPT2', 'LPT3', 'LPT4', 'LPT5', 'LPT6', 'LPT7', 'LPT8', 'LPT9'
        ]

        if sanitized.upper() in reserved_names:
            sanitized = f"_{sanitized}"

        return sanitized

    @staticmethod
    def format_duration(seconds: float) -> str:
        """格式化时间持续时长"""
        if seconds < 60:
            return f"{seconds:.1f}s"
        elif seconds < 3600:
            minutes = int(seconds // 60)
            remaining_seconds = int(seconds % 60)
            return f"{minutes}m {remaining_seconds}s"
        else:
            hours = int(seconds // 3600)
            minutes = int((seconds % 3600) // 60)
            return f"{hours}h {minutes}m"

    @staticmethod
    def get_terminal_size() -> Tuple[int, int]:
        """获取终端尺寸"""
        try:
            import shutil
            size = shutil.get_terminal_size()
            return size.columns, size.lines
        except:
            return 80, 24  # 默认尺寸

    @staticmethod
    def wrap_text(text: str, width: int) -> List[str]:
        """文本换行处理"""
        if not text:
            return [""]

        import textwrap
        wrapped_lines = []

        for line in text.split('\n'):
            if len(line) <= width:
                wrapped_lines.append(line)
            else:
                wrapped_lines.extend(textwrap.wrap(line, width=width))

        return wrapped_lines

    @staticmethod
    def is_chinese_text(text: str) -> bool:
        """检查文本是否包含中文字符"""
        import re
        chinese_pattern = r'[\u4e00-\u9fff]'
        return bool(re.search(chinese_pattern, text))

    @staticmethod
    def count_asian_characters(text: str) -> int:
        """计算亚洲字符数量"""
        import re
        # 包括中文、日文、韩文字符
        asian_pattern = r'[\u4e00-\u9fff\u3040-\u309f\u30a0-\u30ff\uac00-\ud7af]'
        return len(re.findall(asian_pattern, text))


import zipfile
from pathlib import Path


def decompress_zip_to_path(zip_file_path: Path, decompress_path: Path):
    decompress_path.mkdir(exist_ok=True)
    # 解压docx文件到暂存文件夹中
    with zipfile.ZipFile(zip_file_path, 'r') as zipf:
        # 提取所有文件
        zipf.extractall(decompress_path)


def compress_to_zip_file(compress_path: Path, zip_file_path: Path):
    if compress_path.is_dir():
        with zipfile.ZipFile(zip_file_path, 'w', zipfile.ZIP_DEFLATED) as zipf:
            # 遍历文件夹中的所有文件和子文件夹
            for root, _, files in compress_path.walk():
                for file in files:
                    file_path = root / file
                    # 获取文件在压缩文件中的相对路径
                    relative_file_path = file_path.relative_to(compress_path)
                    # 将文件添加到压缩文件中
                    zipf.write(file_path, relative_file_path)
    else:
        with zipfile.ZipFile(zip_file_path, 'w', zipfile.ZIP_DEFLATED) as zipf:
            zipf.write(compress_path)


def replace_in_zip_file(
    src_zip_file_path: Path, dst_zip_file_path: Path,
    content: dict[str, str],
):
    with (
        zipfile.ZipFile(src_zip_file_path, 'r') as zin,
        zipfile.ZipFile(dst_zip_file_path, 'w') as zout,
    ):
        # 遍历原始 ZIP 中的所有文件
        for item in zin.infolist():
            # 如果是目标文件，替换为新内容
            if item.filename in content:
                zout.writestr(item, content[item.filename])
            else:  # 否则直接复制
                zout.writestr(item, zin.read(item.filename))



import fnmatch
from collections import defaultdict
from pathlib import Path
from typing import Callable
from ModuleFolders.Infrastructure.Cache.CacheItem import CacheItem
from ModuleFolders.Infrastructure.Cache.CacheProject import CacheProject
from ModuleFolders.Domain.FileReader import ReaderUtil
from ModuleFolders.Domain.FileReader.BaseReader import BaseSourceReader
from ModuleFolders.Domain.FileReader.ReaderUtil import make_final_detect_text


class DirectoryReader:
    def __init__(self, create_reader: Callable[[], BaseSourceReader], exclude_rules: list[str]):
        self.create_reader = create_reader  # 工厂函数
        self.exclude_files = set()
        self.exclude_paths = set()
        self._update_exclude_rules(exclude_rules)

    def _update_exclude_rules(self, exclude_rules):
        self.exclude_files.update({rule for rule in exclude_rules if "/" not in rule})
        self.exclude_paths.update({rule for rule in exclude_rules if "/" in rule})

    def is_exclude(self, file_path: Path, source_directory: Path):
        if any(fnmatch.fnmatch(file_path.name, rule) for rule in self.exclude_files):
            return True

        # 只有在 source_directory 有效时才计算相对路径
        if source_directory and source_directory.is_dir():
            try:
                rel_path_str = str(file_path.relative_to(source_directory))
                if any(fnmatch.fnmatch(rel_path_str, pattern) for pattern in self.exclude_paths):
                    return True
            except ValueError:
                # 如果不在同一驱动器或路径下，会引发ValueError，此时认为不排除
                pass
        return False

    # 2025年9月8日增加单文件输入支持
    # 树状读取文件夹内同类型文件
    def read_source_directory(self, input_path: Path) -> CacheProject:
        """
        读取文件夹或单个文件，检测每个文件的编码，并在最后设置项目的默认编码。
        此函数现在支持单个文件路径和目录路径。
        Args:
            input_path: 源文件或目录的路径
        Returns:
            CacheProject: 包含项目信息和文件内容
        """
        cache_project = CacheProject()  # 项目头信息
        text_index = 1  # 文本索引
        # 语言统计：{file_path: {lang_code: (count, total_confidence)}}
        language_stats = defaultdict(lambda: defaultdict(lambda: [0, 0.0]))
        file_valid_items_count = defaultdict(int)
        source_texts = defaultdict(list[str])

        # 根据路径类型确定要处理的文件列表和基础目录
        files_to_process = []
        base_directory = None

        # 如果是目录，递归获取所有文件
        if input_path.is_dir():
            base_directory = input_path
            for root, _, files in base_directory.walk():
                for file in files:
                    files_to_process.append(root / file)
        # 如果是单个文件，直接处理该文件
        elif input_path.is_file():
            base_directory = input_path.parent
            files_to_process.append(input_path)
        # 如果路径不存在或不是文件/目录，则返回空项目
        else:
            print(f"Warning: Input path '{input_path}' does not exist or is not a file/directory.")
            return cache_project

        # 创建reader实例
        with self.create_reader() as reader:
            self._update_exclude_rules(reader.exclude_rules)
            cache_project.project_type = reader.get_project_type()

            # 遍历预先生成的文件列表
            for file_path in files_to_process:
                # 检查是否被排除，以及是否是目标类型文件
                if not self.is_exclude(file_path, base_directory) and reader.can_read(file_path):

                    # 读取单个文件的文本信息，并添加其他信息
                    cache_file = reader.read_source_file(file_path)
                    # 空文件跳过
                    if not cache_file:
                        continue

                    # 使用 base_directory 计算相对路径
                    cache_file.storage_path = str(file_path.relative_to(base_directory))
                    cache_file.file_project_type = reader.get_file_project_type(file_path)

                    for item in cache_file.items:
                        item.text_index = text_index
                        item.model = 'none'
                        text_index += 1

                        # 统计每行的语言信息
                        lang_code = item.lang_code
                        # 只统计检测到有效语言代码的item行
                        if lang_code:
                            lang_confidence = lang_code[1]
                            # 更新语言统计：[计数, 累计置信度]
                            stats = language_stats[cache_file.storage_path][lang_code[0]]
                            stats[0] += 1  # 增加计数
                            stats[1] += lang_confidence  # 累加置信度
                            # 累计有效项目总数
                            file_valid_items_count[cache_file.storage_path] += 1
                            # 添加行至后续使用
                            final_detect_text = make_final_detect_text(item)
                            if final_detect_text:
                                source_texts[cache_file.storage_path].append(final_detect_text)

                    # 只有当文件有有效内容时才添加到项目中
                    if cache_file.items:
                        cache_project.add_file(cache_file)
                        # 补充缺失的字典项（仅对已添加到项目的文件）
                        if not language_stats[cache_file.storage_path]:
                            language_stats[cache_file.storage_path] = defaultdict(lambda: [0, 0.0])

        # 处理语言统计结果
        language_counter = defaultdict(list)
        low_confidence_language_counter = defaultdict(list)

        for file_path, lang_stats in language_stats.items():
            # 只有存在有效项目的文件才进行处理
            current_file_items_count = file_valid_items_count[file_path]
            if current_file_items_count > 0:
                high_threshold = max(current_file_items_count * 0.1, min(current_file_items_count, 3))  # 有效项目总数的10%
                mid_threshold = max(current_file_items_count * 0.05, min(current_file_items_count, 2))  # 有效项目总数的5%
                low_threshold = max(current_file_items_count * 0.01, 1)  # 有效项目总数的1%

                # 先计算所有语言的平均置信度
                all_langs = []
                for lang, (count, total_confidence) in lang_stats.items():
                    avg_confidence = total_confidence / count
                    all_langs.append((lang, count, avg_confidence))

                # 按出现次数降序排序，相同次数按置信度降序排序
                sorted_langs = sorted(all_langs, key=lambda x: (-x[1], -x[2]))

                # 筛选高置信度语言
                high_confidence_langs = []
                for lang, count, avg_confidence in sorted_langs:
                    # 应用筛选条件：次数超过阈值且平均置信度大于等于0.82
                    if count >= high_threshold and avg_confidence >= 0.82 or \
                            count >= mid_threshold and avg_confidence >= 0.92 or \
                            count >= low_threshold and avg_confidence >= 0.96:
                        high_confidence_langs.append((lang, count, avg_confidence))

                # 获取有效语言列表
                hc_langs_set = {lang for lang, count, avg_confidence in high_confidence_langs}

                # 如果到这里了还没有high_confidence_langs的结果，使用mp对所有有效文字进行检测
                if not high_confidence_langs:
                    if len(source_texts[file_path]) > 0:
                        mp_langs, mp_score, _ = ReaderUtil.detect_language_with_mediapipe(
                            [CacheItem(source_text='\n'.join(source_texts[file_path]))], 0, None
                        )[0]
                        if mp_score >= 0.82:
                            # 添加到language_counter
                            language_counter[file_path] = [(mp_langs[0], len(source_texts[file_path]), mp_score)]
                            # 添加到 hc_langs_set
                            hc_langs_set.add(mp_langs[0])
                    else:
                        language_counter[file_path] = [('un', len(source_texts[file_path]), -1.0)]
                else:
                    language_counter[file_path] = high_confidence_langs

                # 筛选低置信度语言
                low_confidence_langs = []
                for lang, count, avg_confidence in sorted_langs:
                    # 应用筛选条件：出现次数小于高置信度或者平均置信度小于0.82
                    # (count < high_threshold or avg_confidence < 0.82) and
                    if lang not in hc_langs_set:
                        low_confidence_langs.append((lang, count, avg_confidence))

                if low_confidence_langs:
                    low_confidence_language_counter[file_path] = low_confidence_langs
            else:
                language_counter[file_path] = [('un', 0, -1.0)]

        # 处理未出现在language_counter中的文件
        valid_file_paths = set(file_valid_items_count.keys())
        files_in_counter = set(language_counter.keys())
        missing_files = valid_file_paths - files_in_counter

        # 为未统计到有效语言的文件添加默认值
        for file_path in missing_files:
            # un表示未知语言
            language_counter[file_path] = [('un', 0, -1.0)]

        # 为对应的CacheFile添加语言统计属性（添加安全检查）
        for file_path, langs in language_counter.items():
            cache_file = cache_project.get_file(file_path)
            if cache_file:
                cache_file.language_stats = langs
            else:
                print(f"Warning: File '{file_path}' not found in cache_project when setting language_stats")

        # 添加低置信度语言统计（添加安全检查）
        for file_path, langs in low_confidence_language_counter.items():
            cache_file = cache_project.get_file(file_path)
            if cache_file:
                cache_file.lc_language_stats = langs
            else:
                print(f"Warning: File '{file_path}' not found in cache_project when setting lc_language_stats")

        # 释放语言检测器
        ReaderUtil.close_lang_detector()

        # 自动生成工程名字
        self._generate_project_name(cache_project)

        return cache_project

    # 自动生成工程名字方法
    def _generate_project_name(self, cache_project: CacheProject):
        """
        根据读取的文件列表，自动生成工程名字。
        """
        # 配置的参数
        CHARS_PER_FILENAME = 5
        SEPARATOR = '&&'
        MAX_FILES_FOR_NAME = 4

        # 从字典中获取所有 CacheFile 对象，并转换为列表
        files_list = list(cache_project.files.values())

        if not files_list:
            cache_project.project_name = "EmptyProject"
            return

        if len(files_list) == 1:
            # 单个文件：直接使用文件名（不含扩展名）
            # 使用列表索引 [0] 来访问第一个元素
            # 当输入是单个文件时, storage_path 就是文件名
            file_path = Path(files_list[0].storage_path)
            cache_project.project_name = file_path.stem
        else:
            # 多个文件：组合文件名
            name_parts = []
            # 对列表进行切片
            for cache_file in files_list[:MAX_FILES_FOR_NAME]:
                # storage_path 是相对路径，需要从中获取文件名
                file_stem = Path(cache_file.storage_path).stem
                name_parts.append(file_stem[:CHARS_PER_FILENAME])

            cache_project.project_name = SEPARATOR.join(name_parts)



"""
Provider Fingerprint - API 特性指纹记录系统

功能：
- 记录各 API Provider 的特性支持情况
- 自动检测并标记不支持的功能（缓存、流式等）
- 下次启动时静默降级，避免重复报错
"""

import hashlib
import threading
from typing import Optional, Dict, Any
from enum import Enum

from ModuleFolders.Base.Base import Base


class FeatureSupport(Enum):
    """功能支持状态"""
    UNKNOWN = "unknown"      # 未知，需要探测
    SUPPORTED = "supported"  # 支持
    UNSUPPORTED = "unsupported"  # 不支持


class ProviderFingerprint(Base):
    """Provider 特性指纹管理器"""

    _instance = None
    _lock = threading.Lock()

    def __new__(cls):
        if cls._instance is None:
            with cls._lock:
                if cls._instance is None:
                    cls._instance = super().__new__(cls)
                    cls._instance._initialized = False
        return cls._instance

    def __init__(self):
        if self._initialized:
            return
        super().__init__()
        self._fingerprints: Dict[str, Dict[str, Any]] = {}
        self._load_fingerprints()
        self._initialized = True

    def _get_provider_key(self, api_url: str) -> str:
        """生成 Provider 唯一标识"""
        # 提取主机部分作为 key
        from urllib.parse import urlparse
        parsed = urlparse(api_url)
        host = parsed.netloc or parsed.path.split('/')[0]
        return hashlib.md5(host.encode()).hexdigest()[:12]

    def _load_fingerprints(self) -> None:
        """从配置加载指纹数据"""
        try:
            config = self.load_config()
            self._fingerprints = config.get("provider_fingerprints", {})
        except Exception:
            self._fingerprints = {}

    def _save_fingerprints(self) -> None:
        """保存指纹数据到配置"""
        try:
            config = self.load_config()
            config["provider_fingerprints"] = self._fingerprints
            self.save_config(config)
        except Exception as e:
            self.warning(f"Failed to save provider fingerprints: {e}")

    def get_cache_support(self, api_url: str) -> FeatureSupport:
        """获取 Provider 的缓存支持状态"""
        key = self._get_provider_key(api_url)
        fp = self._fingerprints.get(key, {})
        status = fp.get("cache_support", FeatureSupport.UNKNOWN.value)
        return FeatureSupport(status)

    def set_cache_support(self, api_url: str, supported: bool) -> None:
        """设置 Provider 的缓存支持状态"""
        key = self._get_provider_key(api_url)
        if key not in self._fingerprints:
            self._fingerprints[key] = {"api_url": api_url}

        status = FeatureSupport.SUPPORTED if supported else FeatureSupport.UNSUPPORTED
        self._fingerprints[key]["cache_support"] = status.value
        self._save_fingerprints()

    def get_stream_support(self, api_url: str, model: str) -> FeatureSupport:
        """获取 Provider + Model 的流式支持状态"""
        key = self._get_provider_key(api_url)
        fp = self._fingerprints.get(key, {})
        stream_models = fp.get("stream_models", {})
        status = stream_models.get(model, FeatureSupport.UNKNOWN.value)
        return FeatureSupport(status)

    def set_stream_support(self, api_url: str, model: str, supported: bool) -> None:
        """设置 Provider + Model 的流式支持状态"""
        key = self._get_provider_key(api_url)
        if key not in self._fingerprints:
            self._fingerprints[key] = {"api_url": api_url}

        if "stream_models" not in self._fingerprints[key]:
            self._fingerprints[key]["stream_models"] = {}

        status = FeatureSupport.SUPPORTED if supported else FeatureSupport.UNSUPPORTED
        self._fingerprints[key]["stream_models"][model] = status.value
        self._save_fingerprints()

    def should_use_cache(self, api_url: str) -> bool:
        """判断是否应该使用缓存功能"""
        status = self.get_cache_support(api_url)
        # 未知状态默认尝试使用
        return status != FeatureSupport.UNSUPPORTED

    def mark_cache_unsupported(self, api_url: str, error_msg: str) -> None:
        """标记 Provider 不支持缓存（基于错误信息）"""
        from ModuleFolders.Infrastructure.LLMRequester.ErrorClassifier import ErrorClassifier

        if ErrorClassifier.is_cache_related_error(error_msg):
            self.set_cache_support(api_url, False)
            self.info(f"Provider fingerprint updated: cache disabled for {api_url}")

    def get_fingerprint_summary(self, api_url: str) -> Dict[str, Any]:
        """获取 Provider 的完整指纹摘要"""
        key = self._get_provider_key(api_url)
        return self._fingerprints.get(key, {})

    def clear_fingerprint(self, api_url: str) -> None:
        """清除指定 Provider 的指纹（用于重置）"""
        key = self._get_provider_key(api_url)
        if key in self._fingerprints:
            del self._fingerprints[key]
            self._save_fingerprints()

    def clear_all_fingerprints(self) -> None:
        """清除所有指纹数据"""
        self._fingerprints = {}
        self._save_fingerprints()



import re
from types import SimpleNamespace

from ModuleFolders.Base.Base import Base
from ModuleFolders.Service.TaskExecutor import TranslatorUtil
from ModuleFolders.Infrastructure.TaskConfig.TaskConfig import TaskConfig
from ModuleFolders.Domain.PromptBuilder.PromptBuilder import PromptBuilder

class PromptBuilderLocal(Base):

    def __init__(self) -> None:
        super().__init__()

    # 获取默认系统提示词，优先从内存中读取，如果没有，则从文件中读取
    def get_system_default(config: TaskConfig) -> str:
        if getattr(PromptBuilderLocal, "local_system_zh", None) == None:
            with open("./Resource/Prompt/Local/local_system_zh.txt", "r", encoding = "utf-8") as reader:
                PromptBuilderLocal.local_system_zh = reader.read().strip()
        if getattr(PromptBuilderLocal, "local_system_en", None) == None:
            with open("./Resource/Prompt/Local/local_system_en.txt", "r", encoding = "utf-8") as reader:
                PromptBuilderLocal.local_system_en = reader.read().strip()


        # 如果输入的是字典，则转换为命名空间
        if isinstance(config, dict):
            namespace = SimpleNamespace()
            for key, value in config.items():
                setattr(namespace, key, value)
            config = namespace


        # 构造结果
        if config == None:
            result = PromptBuilderLocal.local_system_zh
        elif  config.target_language in ("chinese_simplified", "chinese_traditional"):
            result = PromptBuilderLocal.local_system_zh
        elif config.target_language not in ("chinese_simplified", "chinese_traditional"):
            result = PromptBuilderLocal.local_system_en

        return result

    # 获取系统提示词
    def build_system(config: TaskConfig, source_lang: str) -> str:

        # 获取默认系统提示词
        PromptBuilderLocal.get_system_default(config)

        en_sl, source_language, en_tl, target_language = TranslatorUtil.get_language_display_names(source_lang, config.target_language)

        # 构造结果
        if config == None:
            result = PromptBuilderLocal.local_system_zh
        elif config.target_language in ("chinese_simplified", "chinese_traditional"):
            result = PromptBuilderLocal.local_system_zh
        elif config.target_language not in ("chinese_simplified", "chinese_traditional"):
            result = PromptBuilderLocal.local_system_en
            source_language = en_sl
            target_language = en_tl

        return result.replace("{source_language}", source_language).replace("{target_language}", target_language).strip()

    # 构造术语表
    def build_glossary_prompt(config: TaskConfig, input_dict: dict) -> str:
        # 将输入字典中的所有值合并为一个字符串，方便正则全局匹配
        full_text = "\n".join(input_dict.values())

        # 筛选并处理匹配的条目
        result = []
        seen_keys = set() # 用于去重 (匹配到的实际原文, 译文)

        for v in config.prompt_dictionary_data:
            src = v.get("src", "")
            if not src:
                continue

            try:
                # 编译正则表达式，忽略大小写以保持与原逻辑一致的宽松匹配
                pattern = re.compile(src, re.IGNORECASE)

                # 查找所有匹配项 (set去重，处理同一词在文中多次出现的情况)
                found_texts = set(m.group() for m in pattern.finditer(full_text))

                # 如果正则匹配到了内容 (例如正则 (A|B) 匹配到了 A 和 B，这里会循环两次)
                for match_text in found_texts:
                    if not match_text: continue

                    # 使用 (实际匹配文本, 译文) 作为唯一键进行去重
                    key = (match_text, v.get("dst"))
                    if key not in seen_keys:
                        # 复制元数据，并将 src 替换为实际匹配到的原文文本
                        new_entry = v.copy()
                        new_entry["src"] = match_text
                        result.append(new_entry)
                        seen_keys.add(key)

            except re.error:
                # 如果正则编译失败（非合法正则），回退到普通字符串包含判断
                if src.lower() in full_text.lower():
                    key = (src, v.get("dst"))
                    if key not in seen_keys:
                        result.append(v)
                        seen_keys.add(key)

        # 数据校验
        if len(result) == 0:
            return ""

        # 避免空的默认内容
        if len(result) == 1 and (result[0]["src"] == ""):
            return ""

        # 初始化变量，以免出错
        glossary_prompt_lines = []

        if config.target_language in ("chinese_simplified", "chinese_traditional"):
            # 添加开头
            glossary_prompt_lines.append(
                "\n###术语表"
                + "\n" + "原文|译文|备注"
            )

            # 添加数据
            for v in result:
                glossary_prompt_lines.append(f"{v.get("src")}|{v.get("dst")}|{v.get("info") if v.get("info") != "" else " "}")

        else:
            # 添加开头
            glossary_prompt_lines.append(
                "\n###Glossary"
                + "\n" + "Original Text|Translation|Remarks"
            )

            # 添加数据
            for v in result:
                glossary_prompt_lines.append(f"{v.get("src")}|{v.get("dst")}|{v.get("info") if v.get("info") != "" else " "}")


        # 拼接成最终的字符串
        glossary_prompt = "\n".join(glossary_prompt_lines)

        return glossary_prompt

    # 生成信息结构 - LocalLLM
    def generate_prompt_LocalLLM(config,  source_text_dict: dict, previous_text_list: list[str], source_lang, rag_context: str = "") -> tuple[list[dict], str, list[str]]:
        # 储存指令
        messages = []
        # 储存额外日志
        extra_log = []

        # 基础提示词
        system = PromptBuilderLocal.build_system(config, source_lang)

        # 如果有 RAG 上下文，注入到系统中
        if rag_context:
            if config.target_language in ("chinese_simplified", "chinese_traditional"):
                system += f"\n\n### 相关历史上下文（供参考）：\n{rag_context}\n"
            else:
                system += f"\n\n### Relevant Historical Context (for reference):\n{rag_context}\n"
            extra_log.append(f"RAG Context added:\n{rag_context}")

        # 术语表
        if config.prompt_dictionary_switch == True:
            result = PromptBuilderLocal.build_glossary_prompt(config, source_text_dict)
            if result != "":
                system = system + "\n" + result
                extra_log.append(result)

        # 构建待翻译文本
        source_text = PromptBuilder.build_source_text(config,source_text_dict)
        pre_prompt = PromptBuilder.build_userQueryPrefix(config) # 用户提问前置文本
        source_text_str = f"{pre_prompt}<textarea>\n{source_text}\n</textarea>"


        # 构建用户提问信息
        messages.append(
            {
                "role": "user",
                "content": source_text_str,
            }
        )


        return messages, system, extra_log


import re




# 检查多行文本回复内容行数是否正确
def check_multiline_text( source_text_dict, translated_dict):
    """
    检查输入字典中的多行文本块是否正确翻译，包括行数和每行内容的完整性。

    Args:
        source_text_dict (dict): 源文本字典，key为字符数字，value为文本。
        input_dict (dict): 输入的字典，key为字符数字，value为文本。

    Returns:
        bool: 所有多行文本块检查通过返回True，否则返回False。
    """

    if (len(source_text_dict) == 1) and (len(translated_dict) == 1):
        return True  # 一行就不检查了

    # 获取排序后的key，确保按数字顺序检查
    keys = sorted(source_text_dict.keys(), key=int)

    for key in keys:
        # 检查key是否存在于输入字典中
        if key not in translated_dict:
            return False

        source_text = source_text_dict[key]
        translated_text = translated_dict[key]

        # 去除头尾的空格和换行符
        trimmed_source_text = source_text.strip()
        trimmed_translated_text = translated_text.strip()

        # 在处理过的文本上计算文本内的换行符数量
        source_newlines = trimmed_source_text.count('\n')
        # 检查原文中的转义换行符
        source_newlines += trimmed_source_text.count('\\n')

        translated_newlines = trimmed_translated_text.count('\n')
        translated_newlines += trimmed_translated_text.count('\\n')

        # 检查换行符数是否匹配，要放在外面进行比较，因为source_text可能没有换行符，而译文就有
        if source_newlines != translated_newlines:
            return False

        # 如果源文本包含换行符，则需要检查每一行
        if '\n' in source_text: 
            # 分割成行
            source_lines = source_text.split('\n')
            input_lines = translated_text.split('\n')

            # 检查每一行是否都有实际内容（除了序号和格式标记外）
            for i, line in enumerate(input_lines):
                # 移除序号部分（如"1.2.,"）
                content_after_prefix = re.sub(r'^\s*\d+\.\d+\.,\s*', '', line).strip()

                # 如果回复行为空
                if not content_after_prefix:
                    # 检查对应的源文本行是否也为空
                    if not source_lines[i].strip():
                        continue
                    else:
                        # 源文本有内容但翻译没有，不通过检查
                        return False

    return True  # 所有检查都通过



# 检查是否成功保留全部的占位符
def check_placeholders_exist(placeholder_info: dict, translated_dict: dict) -> bool:
    """
    检查 translated_dict 中的文本是否包含 placeholder_info 中定义的所有占位符。

    Args:
        placeholder_info: 包含占位符信息的字典。
                        键是段落 ID (字符串), 值是包含占位符字典的列表。
                        每个占位符字典包含 "placeholder" (占位符字符串) 键。
        translated_dict: 包含文本内容的字典。
                    键是段落 ID (字符串), 值是对应的文本字符串。

    Returns:
        如果所有定义的占位符都存在于其对应的文本段落中，则返回 True；
        否则返回 False。
    """
    # 非空检查
    if not placeholder_info:
        return True

    # 遍历占位符信息字典中的每个段落 ID 和对应的占位符列表
    for text_id, placeholder_list in placeholder_info.items():
        # 检查文本内容字典中是否存在对应的段落 ID
        if text_id not in translated_dict:
            return False

        # 获取对应段落的文本内容
        segment_text = translated_dict[text_id]

        # 如果当前段落没有需要检查的占位符，则跳到下一个段落
        if not placeholder_list:
            continue

        # 遍历当前段落需要检查的所有占位符
        for placeholder_data in placeholder_list:
            placeholder = placeholder_data.get("placeholder")

            # 确保 'placeholder' 键存在且值是字符串
            if not placeholder or not isinstance(placeholder, str):
                continue 

            # 核心检查：占位符是否存在于文本中
            if placeholder not in segment_text:
                return False  # 发现一个缺失，即可确定结果为 False，提前退出

    return True



# 检查两个字典是否完全相同，即返回了原文
def check_dicts_equal(dict1, dict2):

    # 不检测双行及以下
    if len(dict1) >=3 :
        i = 0
        s = 0
        for key, value in dict1.items():
            value2 = dict2[key]

            # 将字符串转换为集合形式
            set1 = set(value)
            set2 = set(value2)

            
            # 定义日本汉字的Unicode范围（这是一个大致范围，可能需要调整）
            kanji_start = 0x4E00
            kanji_end = 0x9FFF
            # 剔除原文集合中的汉字
            set1_test = {char for char in set1 if not (kanji_start <= ord(char) <= kanji_end)}
            #set2 = {char for char in set2 if not (kanji_start <= ord(char) <= kanji_end)}
            # 如果原文集合为空，说明原文全是汉字，则跳过此行的计算
            if not set1_test:
                continue


            # 计算交集和并集的大小
            intersection_size = len(set1.intersection(set2))
            union_size = len(set1.union(set2))

            # 计算单个文本行的Jaccard相似系数
            similarity = intersection_size / union_size

            #累加与累计
            i = i + 1
            s = s + similarity

        # 计算总体相似度，并防止除到0
        result = s / i if i != 0 else 0

        if (result>= 0.85):
            return False
        else:
            return True

    else:
        return True



# 检查残留原文的算法
def detecting_remaining_original_text(dictA, dictB, language, target_language=None):

    # 使用复制变量，避免影响到原变量
    dict_src = dictA.copy()
    dict_dst = dictB.copy()

    # 考量到代码文本，不支持的语言不作检查
    if language not in ("japanese","korean","chinese_simplified","chinese_traditional"):
        return True

    # 中文→日文时跳过检测，因为日文本身包含大量汉字
    if language in ("chinese_simplified", "chinese_traditional") and target_language and target_language.lower() == "japanese":
        return True

    # 避免检查单或者少行字典
    if len(dict_src) <=1 :
        return True

    # 不同语言的通用标点符号字符集
    punctuation_pattern_sets = re.compile(
        r'['
        r'\u3000-\u303F'   # CJK符号和标点
        r'\uFF01-\uFF9F'   # 全角ASCII、半角片假名和日文标点
        r'\u0500-\u052F'   # 俄语扩展字符（含标点）
        r']+', re.UNICODE  # 使用 + 来匹配一个或多个字符
    )

    # 特定的无法过滤的标点符号集
    punctuation_list = ['(', ')', '・', '?', '？', '『', '』', '（', '）', '＜', '＞', '·', '～', 'ー', '@', '＠', '·', '.', '♡', '…', '。', '！', '、', '，', 'の']  

    # 定义不同语言的文本字符集对应的正则表达式
    patterns_language = {
        'japanese': re.compile(
            r'['
            r'\u3041-\u3096'  # 平假名
            r'\u30A0-\u30FF'  # 片假名
            r']+', re.UNICODE
        ),
        'korean': re.compile(
            r'['
            r'\uAC00-\uD7AF'  # 韩文字母
            r']+', re.UNICODE
        ),
        'chinese_simplified': re.compile(
            r'['
            r'\u4E00-\u9FA5'  # 简体汉字
            r']+', re.UNICODE
        ),
        'chinese_traditional': re.compile(
            r'['
            r'\u3400-\u4DBF'  # 扩展A区汉字
            r'\u4E00-\u9FFF'  # 基本汉字
            r'\uF900-\uFAFF'  # 兼容汉字
            r']+', re.UNICODE
        ),
    }

    # 存储计数结果的字典
    count_results = 0

    # 遍历译文字典中的每个键值对
    for key_dst, value_dst in dict_dst.items():

        # 分组提取译文中的指定语言的文本
        text_lsit =  patterns_language.get(language).findall(value_dst)
        # 提取为空内容，则跳过
        if not text_lsit:
            continue

        # 获取对应原文
        text_src = dict_src[key_dst]

        # 检查是否注音文本
        if contains_specific_format_single_comma(text_src):
            print("已过滤原文：" + text_src)
            continue

        # 循环处理，移除译文中的所有标点符号，并进行检查
        for text in text_lsit:
            # 移除标点符号
            text = re.sub(punctuation_pattern_sets, '', text)
            text = remove_punctuation(text, punctuation_list)

            # 如果移除后为空，则跳过
            if not text:
                continue

            # 检查是否有原文残留
            if text_src:

                # 检查是否在原文中
                if  text in text_src:
                    count_results += 1        

                # 如果没有，则检查是否有原文的单个在原文中
                else:
                    for char in text:
                        if char in text_src:
                            count_results += 1                 

    # 根据出现次数判断结果
    #print("count_results:", count_results)  # 调试输出
    if  count_results >=1:
        return False

    return True

# 辅助函数
def remove_punctuation(input_string, punctuation_list):
    """
    移除输入字符串中所有属于标点符号列表的字符。

    :param input_string: 要移除标点的字符串。
    :param punctuation_list: 需要移除的标点符号列表。
    :return: 移除了指定标点的新字符串。
    """
    result = ''.join(char for char in input_string if char not in punctuation_list)
    return result

# 辅助函数，检测rpgmaker游戏的人名注音文本
def contains_specific_format_single_comma(text: str) -> bool:
    """
    判断输入的文本是否含有特定格式的子字符串。
    格式为：\r[必须有任意文本,可能有文本]，且方括号内只能有一个逗号。
    例如：\r[助平,すけべい] 或 \r[くん,] 或 [P1][くん,]

    Args:
        text (str): 需要检查的输入文本。

    Returns:
        bool: 如果文本中含有该格式的子字符串，则返回 True，否则返回 False。
    """
    pattern = r"(?:\\r|\[P\d+\])?\[[^,\]]+,[^,\]]*\]"

    # 使用 re.search 在文本中查找匹配项
    if re.search(pattern, text):
        return True
    else:
        return False


# 回复格式检查函数
def check_reply_format(source_text_dict: dict, response_dict: dict) -> bool:
    """
    严格检查回复的格式是否与build_source_text方法构建的原文格式完全一致。

    注意：此函数检查的是经过 ResponseExtractor.text_extraction 处理后的格式！

    格式要求：
    - 单行文本：{序号}.{内容}
      例如：1.こんにちは

    - 多行文本（处理后）：{序号}.{倒序子序号}.,{内容}\n{序号}.{倒序子序号-1}.,{内容}\n...
      例如：1.3.,第三行\n1.2.,第二行\n1.1.,第一行

    Args:
        source_text_dict (dict): 源文本字典。
        response_dict (dict): 解析后的回复文本字典（已经过text_extraction处理）。

    Returns:
        bool: 如果格式完全匹配则返回 True，否则返回 False。
    """
    for key, source_text in source_text_dict.items():
        response_text = response_dict.get(key)

        # 如果对应的回复不存在，则格式错误
        if response_text is None:
            return False

        main_index = int(key) + 1  # 主序号（从1开始）
        is_source_multiline = '\n' in source_text

        if is_source_multiline:
            # === 多行文本格式严格检查（处理后的格式）===
            source_lines = source_text.split('\n')
            total_lines = len(source_lines)

            # 检查response_text是否也是多行
            response_lines = response_text.split('\n')

            # 1. 行数必须一致
            if len(response_lines) != total_lines:
                return False

            # 2. 检查每一行的格式：{主序号}.{子序号}.,{内容}
            expected_sub_numbers = list(range(total_lines, 0, -1))  # 倒序：[3, 2, 1]

            for i, (response_line, expected_sub_num) in enumerate(zip(response_lines, expected_sub_numbers)):
                # 期望的行前缀格式："{主序号}.{子序号}.,"
                expected_line_prefix = f"{main_index}.{expected_sub_num}.,"

                if not response_line.startswith(expected_line_prefix):
                    return False

            # 3. 额外验证：确保子序号确实是连续倒序的
            # 提取所有子序号
            sub_number_pattern = re.compile(rf'^{main_index}\.(\d+)\.,')
            found_sub_numbers = []

            for line in response_lines:
                match = sub_number_pattern.match(line)
                if match:
                    found_sub_numbers.append(int(match.group(1)))
                else:
                    return False  # 某行格式不对

            # 验证子序号是否为期望的倒序序列
            if found_sub_numbers != expected_sub_numbers:
                return False

        else:
            # === 单行文本格式严格检查 ===
            # 必须以 "{主序号}." 开头
            expected_prefix = f"{main_index}."
            if not response_text.startswith(expected_prefix):
                return False

    return True


def check_untranslated_lines(source_text_dict, response_dict, source_lang, target_lang, retry_count=0, retry_limit=3, reduction_threshold=0.5):
    """
    检测译文中是否存在未翻译的行
    通过对比原文和译文的源语言字符变化率来判断

    参数:
        source_text_dict: 原文字典 {序号: 原文}
        response_dict: 译文字典 {序号: 译文}
        source_lang: 源语言代码
        target_lang: 目标语言代码
        retry_count: 当前重试次数
        retry_limit: 最大重试次数阈值
        reduction_threshold: 源语言字符减少率阈值（低于此值视为未翻译）

    返回:
        True: 检查通过（已翻译或超过重试次数）
        False: 检查不通过（疑似未翻译，需要重试）
    """
    # 超过重试次数，直接放行
    if retry_count >= retry_limit:
        return True

    # 如果重试次数阈值为0，禁用此检测
    if retry_limit <= 0:
        return True

    # 语言特有字符正则
    lang_patterns = {
        'japanese': r'[\u3041-\u3096\u30A0-\u30FF]',  # 假名
        'korean': r'[\uAC00-\uD7AF]',  # 韩文
        'english': r'[a-zA-Z]',  # 英文
        'russian': r'[\u0400-\u04FF]',  # 俄文
    }

    source_pattern = lang_patterns.get(source_lang)

    # 不支持的语言组合，跳过检测
    if not source_pattern:
        return True

    # 源语言和目标语言相同，跳过
    if source_lang == target_lang:
        return True

    for key in source_text_dict:
        source = source_text_dict[key].strip()
        response = response_dict.get(key, "").strip()

        # 跳过过短的行（可能是符号、数字等）
        if len(source) < 2:
            continue

        # === 检测1：完全相同 ===
        if source == response:
            return False

        # === 检测2：去除标点空格后相同 ===
        source_clean = re.sub(r'[\s\.\,\!\?\。\、\，\！\？\「\」\『\』\(\)\（\）\~\～\-\—\:\：\;\；\'\"\'\'\"\"]', '', source)
        response_clean = re.sub(r'[\s\.\,\!\?\。\、\，\！\？\「\」\『\』\(\)\（\）\~\～\-\—\:\：\;\；\'\"\'\'\"\"]', '', response)
        if source_clean == response_clean:
            return False

        # === 检测3：源语言字符减少率 ===
        source_chars = len(re.findall(source_pattern, source))
        response_chars = len(re.findall(source_pattern, response))

        # 原文中源语言字符足够多才检测（避免误判混合语言）
        if source_chars > 3:
            reduction_rate = (source_chars - response_chars) / source_chars
            # 减少率低于阈值，疑似未翻译
            if reduction_rate < reduction_threshold:
                return False

    return True


from pathlib import Path
from typing import Callable

from ModuleFolders.Infrastructure.Cache.CacheFile import CacheFile
from ModuleFolders.Infrastructure.Cache.CacheItem import CacheItem
from ModuleFolders.Infrastructure.Cache.CacheProject import ProjectType
from ModuleFolders.Domain.FileOutputer.BaseWriter import (
    BaseBilingualWriter,
    BaseTranslatedWriter,
    OutputConfig,
    PreWriteMetadata,
    BilingualOrder,
)


class TxtWriter(BaseBilingualWriter, BaseTranslatedWriter):
    def __init__(self, output_config: OutputConfig):
        super().__init__(output_config)

    def on_write_bilingual(
        self, translation_file_path: Path, cache_file: CacheFile,
        pre_write_metadata: PreWriteMetadata,
        source_file_path: Path = None,
    ):
        self._write_translation_file(translation_file_path, cache_file, pre_write_metadata, self._item_to_bilingual_line)

    def on_write_translated(
        self, translation_file_path: Path, cache_file: CacheFile,
        pre_write_metadata: PreWriteMetadata,
        source_file_path: Path = None,
    ):
        self._write_translation_file(translation_file_path, cache_file, pre_write_metadata, self._item_to_translated_line)

    def _write_translation_file(
        self, translation_file_path: Path, cache_file: CacheFile,
        pre_write_metadata: PreWriteMetadata,
        item_to_line: Callable[[CacheItem], str],
    ):
        if not cache_file.items:
            translation_file_path.touch()
            return

        # 处理所有项目
        lines = list(map(item_to_line, cache_file.items))

        translation_file_path.write_text("".join(lines), encoding=pre_write_metadata.encoding)

    # 双语版构建
    def _item_to_bilingual_line(self, item: CacheItem):
        line_break = "\n" * max(item.require_extra("line_break") + 1, 1)
        
        # 检查配置并决定输出顺序
        if self.output_config.bilingual_order == BilingualOrder.TRANSLATION_FIRST:
            return (
                f"{item.final_text}\n"
                f"{item.source_text}{line_break}"
            )
        else: # 默认为原文在前
            return (
                f"{item.source_text}\n"
                f"{item.final_text}{line_break}"
            )
        
    # 译文版构建
    def _item_to_translated_line(self, item: CacheItem):
        line_break = "\n" * (item.require_extra("line_break") + 1)

        return f"{item.final_text}{line_break}"

    @classmethod
    def get_project_type(self):
        return ProjectType.TXT


"""
编辑器菜单模块
从 ainiee_cli.py 分离
"""
import os
import time
import rapidjson as json

from rich.console import Console
from rich.panel import Panel
from rich.prompt import Prompt, IntPrompt, Confirm
from rich.table import Table

from ModuleFolders.UserInterface.Editor import TUIEditor

console = Console()


class EditorMenu:
    """编辑器菜单"""

    def __init__(self, host):
        """
        初始化编辑器菜单

        Args:
            host: CLIMenu实例，提供config、i18n等依赖
        """
        self.host = host

    @property
    def config(self):
        return self.host.config

    @property
    def i18n(self):
        return self.host.i18n

    def show(self):
        """显示编辑器菜单（入口方法）"""
        while True:
            self.host.display_banner()
            console.print(Panel(f"[bold]{self.i18n.get('menu_editor') or 'Translation Editor'}[/bold]"))

            # 检查是否有可用的项目
            if not hasattr(self.host, 'cache_manager') or not self.host.cache_manager:
                console.print("[red]No cache manager available. Please run a translation task first.[/red]")
                time.sleep(2)
                break

            # 检查是否有可用的项目缓存
            output_path = self.config.get("label_output_path", "./output")
            cache_file = os.path.join(output_path, "cache", "AinieeCacheData.json")

            if not os.path.exists(cache_file):
                console.print(f"[yellow]{self.i18n.get('editor_no_cache')}[/yellow]")
                console.print(f"[dim]{self.i18n.get('editor_expected_cache')}: {cache_file}[/dim]")
                input("\nPress Enter to continue...")
                break

            # 显示菜单选项
            table = Table(show_header=False, box=None)
            table.add_row("[cyan]1.[/]", self.i18n.get("menu_editor_open_current"))
            table.add_row("[cyan]2.[/]", self.i18n.get("menu_editor_input_path"))
            table.add_row("[cyan]3.[/]", self.i18n.get("menu_editor_scan_projects"))
            table.add_row("[cyan]4.[/]", self.i18n.get("menu_ai_proofread") or "AI自主校对")
            table.add_row("[cyan]5.[/]", self.i18n.get("menu_view_proofread_report") or "查看校对报告")
            console.print(table)
            console.print(f"\n[dim]0. {self.i18n.get('menu_back') or 'Back'}[/dim]")

            choice = IntPrompt.ask(f"\n{self.i18n.get('prompt_select') or 'Select'}",
                                 choices=["0", "1", "2", "3", "4", "5"], show_choices=False)

            if choice == 0:
                break
            elif choice == 1:
                self._start_editor_with_current_project()
            elif choice == 2:
                self._input_project_path_for_editor()
            elif choice == 3:
                self._select_project_for_editor()
            elif choice == 4:
                self.host.ai_proofread_menu.show()
            elif choice == 5:
                self.host.ai_proofread_menu._view_proofread_reports()

    def _start_editor_with_current_project(self):
        """使用当前项目启动编辑器"""
        try:
            # 获取术语表数据
            glossary_data = getattr(self.host, 'prompt_dictionary_data', [])

            # 创建编辑器实例
            editor = TUIEditor(
                cache_manager=self.host.cache_manager,
                config=self.config,
                i18n=self.i18n,
                glossary_data=glossary_data
            )

            # 获取当前项目路径
            project_path = self.config.get("label_output_path", "./output")

            console.print(f"[green]{self.i18n.get('editor_starting')}: {os.path.basename(project_path)}[/green]")
            console.print(f"[dim]{self.i18n.get('editor_quit_tip')}[/dim]")
            time.sleep(1)

            # 启动编辑器
            success = editor.start_editor(project_path)

            if success:
                console.print(f"[green]{self.i18n.get('editor_session_completed')}[/green]")
            else:
                console.print(f"[red]{self.i18n.get('editor_failed_start')}[/red]")

        except Exception as e:
            console.print(f"[red]{self.i18n.get('editor_error_start')}: {e}[/red]")

        input("\nPress Enter to continue...")

    def _input_project_path_for_editor(self):
        """手动输入项目路径进行编辑"""
        try:
            console.print(Panel(f"[bold]{self.i18n.get('editor_input_path_title')}[/bold]"))
            console.print(f"[dim]{self.i18n.get('editor_input_path_tip')}[/dim]")
            console.print(f"[dim]{self.i18n.get('editor_input_path_example')}[/dim]\n")

            # 提示用户输入路径
            project_path = Prompt.ask(self.i18n.get('editor_project_path_prompt'))

            if not project_path or not project_path.strip():
                console.print(f"[yellow]{self.i18n.get('editor_path_empty')}[/yellow]")
                input("\nPress Enter to continue...")
                return

            project_path = project_path.strip()

            # 检查路径是否存在
            if not os.path.exists(project_path):
                console.print(f"[red]{self.i18n.get('editor_path_not_exist')}: {project_path}[/red]")
                input("\nPress Enter to continue...")
                return

            # 检查缓存文件是否存在
            cache_file = os.path.join(project_path, "cache", "AinieeCacheData.json")
            if not os.path.exists(cache_file):
                console.print(f"[red]{self.i18n.get('editor_cache_not_found')}: {cache_file}[/red]")
                console.print(f"[dim]{self.i18n.get('editor_cache_path_tip')}[/dim]")
                input("\nPress Enter to continue...")
                return

            # 分析项目信息
            project_info = self._analyze_cache_file(cache_file)
            if not project_info:
                console.print(f"[red]{self.i18n.get('editor_parse_failed')}[/red]")
                input("\nPress Enter to continue...")
                return

            # 显示项目信息并确认
            console.print(f"\n[green]{self.i18n.get('editor_found_project')}:[/green] {project_info['name']}")
            console.print(f"[dim]{self.i18n.get('editor_path_label')}: {project_info['path']}[/dim]")
            console.print(f"[dim]{self.i18n.get('editor_items_label')}: {project_info['item_count']}[/dim]")
            console.print(f"[dim]{self.i18n.get('editor_size_label')}: {project_info['size']}[/dim]")

            confirm = Confirm.ask(f"\n{self.i18n.get('editor_confirm_open')}", default=True)
            if confirm:
                self._start_editor_with_selected_project(project_info)
            else:
                console.print(f"[yellow]{self.i18n.get('editor_cancelled')}[/yellow]")
                input("\nPress Enter to continue...")

        except KeyboardInterrupt:
            console.print(f"\n[yellow]{self.i18n.get('editor_cancelled')}[/yellow]")
        except Exception as e:
            console.print(f"[red]{self.i18n.get('editor_input_error')}: {e}[/red]")
            input("\nPress Enter to continue...")

    def _select_project_for_editor(self):
        """选择项目缓存进行编辑"""
        try:
            console.print(f"[blue]{self.i18n.get('editor_scanning_cache')}[/blue]")

            # 扫描可用的缓存文件（调用host的方法）
            cache_projects = self.host._scan_cache_files()

            if not cache_projects:
                console.print(f"[yellow]{self.i18n.get('editor_no_cached_projects')}[/yellow]")
                console.print(f"[dim]{self.i18n.get('editor_cache_pattern_tip')}[/dim]")
                input("\nPress Enter to continue...")
                return

            while True:
                self.host.display_banner()
                console.print(Panel(f"[bold]{self.i18n.get('menu_editor_select_project')}[/bold]"))

                # 显示可用的项目
                table = Table(show_header=True, box=None)
                table.add_column("ID", style="cyan", width=4)
                table.add_column(self.i18n.get("editor_project_name"), style="green")
                table.add_column(self.i18n.get("editor_project_path"), style="dim")
                table.add_column(self.i18n.get("editor_item_count"), style="yellow", width=10)
                table.add_column(self.i18n.get("editor_file_size"), style="blue", width=10)

                for i, project in enumerate(cache_projects):
                    table.add_row(
                        f"{i+1}",
                        project["name"],
                        project["path"],
                        str(project["item_count"]),
                        project["size"]
                    )

                console.print(table)
                console.print(f"\n[dim]0. {self.i18n.get('menu_back') or 'Back'}[/dim]")

                try:
                    choice = IntPrompt.ask(f"\n{self.i18n.get('prompt_select') or 'Select'}",
                                         choices=[str(i) for i in range(len(cache_projects) + 1)],
                                         show_choices=False)

                    if choice == 0:
                        break
                    elif 1 <= choice <= len(cache_projects):
                        selected_project = cache_projects[choice - 1]
                        self._start_editor_with_selected_project(selected_project)
                        break

                except (ValueError, KeyboardInterrupt):
                    break

        except Exception as e:
            console.print(f"[red]Error in project selection: {e}[/red]")
            input("\nPress Enter to continue...")

    def _analyze_cache_file(self, cache_file_path):
        """分析缓存文件获取项目信息"""
        try:
            # 获取项目路径（cache文件所在目录的父目录）
            project_path = os.path.dirname(os.path.dirname(cache_file_path))
            project_name = os.path.basename(project_path)

            # 获取文件统计信息
            stat_info = os.stat(cache_file_path)
            file_size = stat_info.st_size
            modified_time = stat_info.st_mtime

            # 尝试解析缓存文件获取条目数量（只统计有翻译内容的）
            item_count = 0
            translated_count = 0
            try:
                with open(cache_file_path, 'r', encoding='utf-8') as f:
                    cache_data = json.load(f)

                # 根据缓存文件结构计算条目数量
                if isinstance(cache_data, dict):
                    # 新格式缓存
                    files_data = cache_data.get("files", {})
                    for file_data in files_data.values():
                        if isinstance(file_data, dict):
                            items = file_data.get("items", [])
                            for item in items:
                                if isinstance(item, dict):
                                    item_count += 1
                                    # 检查是否有翻译内容
                                    translated_text = item.get("translated_text", "") or item.get("polished_text", "")
                                    if translated_text and translated_text.strip():
                                        translated_count += 1
                elif isinstance(cache_data, list) and len(cache_data) > 1:
                    # 旧格式缓存
                    for item in cache_data[1:]:  # 跳过项目信息
                        if isinstance(item, dict):
                            item_count += 1
                            translated_text = item.get("translated_text", "") or item.get("polished_text", "")
                            if translated_text and translated_text.strip():
                                translated_count += 1

            except Exception:
                item_count = 0
                translated_count = 0

            # 如果没有翻译内容，跳过这个项目
            if translated_count == 0:
                return None

            # 格式化文件大小
            if file_size < 1024:
                size_str = f"{file_size} B"
            elif file_size < 1024 * 1024:
                size_str = f"{file_size / 1024:.1f} KB"
            else:
                size_str = f"{file_size / (1024 * 1024):.1f} MB"

            return {
                "name": project_name,
                "path": project_path,
                "cache_file": cache_file_path,
                "item_count": translated_count,  # 显示实际有翻译内容的条目数
                "total_items": item_count,  # 保留总条目数用于调试
                "size": size_str,
                "modified_time": modified_time
            }

        except Exception as e:
            # 如果分析失败，返回None
            return None

    def _start_editor_with_selected_project(self, project_info):
        """使用选定的项目启动编辑器"""
        try:
            # 获取术语表数据
            glossary_data = getattr(self.host, 'prompt_dictionary_data', [])

            # 创建编辑器实例
            editor = TUIEditor(
                cache_manager=self.host.cache_manager,
                config=self.config,
                i18n=self.i18n,
                glossary_data=glossary_data
            )

            console.print(f"[green]{self.i18n.get('editor_starting')}: {project_info['name']}[/green]")
            console.print(f"[dim]{self.i18n.get('editor_project_info')}: {project_info['path']}[/dim]")
            console.print(f"[dim]{self.i18n.get('editor_item_info')}: {project_info['item_count']}[/dim]")
            console.print(f"[dim]{self.i18n.get('editor_quit_tip')}[/dim]")
            time.sleep(1)

            # 启动编辑器
            success = editor.start_editor(project_info['path'])

            if success:
                console.print(f"[green]{self.i18n.get('editor_session_completed')}[/green]")

                # 将此项目添加到最近使用列表
                self._add_to_recent_projects(project_info['path'])
            else:
                console.print(f"[red]{self.i18n.get('editor_failed_start')}[/red]")

        except Exception as e:
            console.print(f"[red]{self.i18n.get('editor_error_start')}: {e}[/red]")
            import traceback
            console.print(f"[dim]Traceback: {traceback.format_exc()}[/dim]")

        input("\nPress Enter to continue...")

    def _add_to_recent_projects(self, project_path):
        """将项目添加到最近使用列表"""
        try:
            recent_projects = self.config.get("recent_projects", [])

            # 移除已存在的相同路径
            if project_path in recent_projects:
                recent_projects.remove(project_path)

            # 添加到列表开头
            recent_projects.insert(0, project_path)

            # 只保留最近的10个项目
            recent_projects = recent_projects[:10]

            # 保存到配置
            self.config["recent_projects"] = recent_projects
            self.host.save_config()

        except Exception:
            pass  # 静默处理错误，不影响主要功能



# # 辅助函数：Softmax
# import os
# import sys
# import time
#
# import numpy as np
# import onnxruntime
# import rich
# from langcodes import Language
# from transformers import AutoTokenizer, AutoConfig
#
#
# def softmax(x, axis=-1):
#     """计算 softmax"""
#     x_max = np.max(x, axis=axis, keepdims=True)
#     exp_x = np.exp(x - x_max)
#     return exp_x / np.sum(exp_x, axis=axis, keepdims=True)
#
#
# class LanguageDetectorONNX:
#     _instance = None
#     _initialized = False
#
#     def __new__(cls, *args, **kwargs):
#         if cls._instance is None:
#             # rich.print("[[green]INFO[/]] 准备 ONNX 文本语言检测器实例中...")
#             cls._instance = super().__new__(cls)
#         return cls._instance
#
#     def __init__(self):
#         if self._initialized:
#             return
#
#         rich.print("[[green]INFO[/]] 正在初始化 ONNX 文本语言检测器实例...")
#         # Record start time
#         start_time = time.time()
#         # 设置模型目录
#         script_dir = os.path.dirname(os.path.abspath(sys.argv[0]))
#         model_path = os.path.join(script_dir, "Resource", "Models", "language_detection-ONNX")
#
#         self.onnx_model_path = os.path.join(model_path, "onnx", "model_int8.onnx")
#         self.tokenizer_path = model_path
#         self.ort_session = None
#         self.tokenizer = None
#         self.id2label = None
#
#         # 使用 GPU加速 (如果可用)
#         available_providers = onnxruntime.get_available_providers()
#         providers_to_use = ['CPUExecutionProvider']  # 默认
#         if 'DmlExecutionProvider' in available_providers:
#             rich.print("[[green]INFO[/]] 检测到有效加速环境，优先使用GPU进行 ONNX 文本语言检测推理!")
#             providers_to_use.insert(0, 'DmlExecutionProvider')
#         else:
#             rich.print("[[green]INFO[/]] 未检测到有效加速环境，默认使用CPU进行 ONNX 文本语言检测推理!")
#
#         try:
#             self.ort_session = onnxruntime.InferenceSession(self.onnx_model_path, providers=providers_to_use)
#             # rich.print(f"[[green]INFO[/]] ONNX session loaded from {self.onnx_model_path}")
#
#             self.tokenizer = AutoTokenizer.from_pretrained(self.tokenizer_path)
#             # rich.print(f"[[green]INFO[/]] Tokenizer loaded from {self.tokenizer_path}")
#
#             try:
#                 config = AutoConfig.from_pretrained(model_path)
#                 self.id2label = {int(k): v for k, v in config.id2label.items()}
#                 # rich.print(f"[[green]INFO[/]] Loaded id2label map from config: {model_path}")
#             except Exception as e:
#                 raise ValueError(
#                     f"Could not load id2label from config '{model_path}': {e}. Please provide id2label_map directly."
#                 )
#
#             if not self.id2label or not isinstance(self.id2label, dict):
#                 raise TypeError("Failed to obtain a valid id2label dictionary.")
#
#             self._initialized = True
#             # 计算加载时间（毫秒）
#             load_time_ms = (time.time() - start_time) * 1000
#             rich.print(f"[[green]INFO[/]] ONNX 文本语言检测器已加载! ({load_time_ms:.2f} ms)")
#
#         except Exception as e:
#             rich.print(f"[[red]ERROR[/]] 加载 ONNX 语言检测器失败: {e}")
#             # 清理，防止处于部分初始化状态
#             self._instance = None  # 确保 instance 也被重置
#             self._initialized = False
#             raise
#
#     @classmethod
#     def release(cls):
#         """
#         释放单例持有的资源并重置其状态。
#         允许后续调用重新初始化单例。
#         """
#         if cls._instance:
#             # rich.print("[[green]INFO[/]] 正在释放 ONNX 文本语言检测器...")
#             if cls._instance.ort_session:
#                 # ONNX Runtime sessions don't have an explicit close/del method in Python API
#                 # that guarantees immediate resource release beyond what GC does.
#                 # Setting to None removes our reference, allowing GC to collect it.
#                 cls._instance.ort_session = None
#                 rich.print("[[green]INFO[/]] ONNX 文本语言检测器已释放!")
#             cls._instance.tokenizer = None
#             cls._instance.id2label = None
#             # 其他需要清理的实例变量也可以在这里设置为 None
#
#             # 最重要的是重置类级别的状态变量
#             cls._instance = None
#             cls._initialized = False
#             # rich.print("[[green]INFO[/]] ONNX 文本语言检测器实例已释放并重置!")
#         else:
#             rich.print("[[green]INFO[/]] No LanguageDetectorONNX singleton instance to release.")
#
#     def predict(self, text):
#         """单文本预测方法"""
#         if not self._initialized or not self.ort_session or not self.tokenizer:
#             if not LanguageDetectorONNX._instance or not LanguageDetectorONNX._initialized:
#                 raise RuntimeError("LanguageDetectorONNX singleton is not properly initialized or has been released.")
#             if not self.ort_session or not self.tokenizer:
#                 raise RuntimeError("LanguageDetectorONNX resources (session/tokenizer) are missing. Was it released?")
#         try:
#             # 调用批处理版本，批大小为1
#             results = self.predict_batch([text])
#             return results[0] if results else None  # predict_batch返回列表，取第一个
#         except Exception as e:
#             print(f"Error during single prediction for text '{text[:50]}...': {e}")
#             return None
#
#     def predict_batch(self, texts: list):  # 新增的批处理预测方法
#         """
#         使用加载的 ONNX 模型批量预测输入文本列表的语言。
#
#         Args:
#             texts (list of str): 需要检测语言的文本列表。
#
#         Returns:
#             list of dict: 每个字典包含对应文本的预测结果:
#                           {'label': str, 'confidence': float, 'top_3_scores': list}
#                           如果某一项预测失败，则对应位置可能为 None 或错误信息字典。
#             None: 如果在批处理过程中发生严重错误。
#         """
#         if not self._initialized or not self.ort_session or not self.tokenizer:
#             # 检查 _instance 是否还存在，如果 release 后实例被设为 None，直接访问 self 上的属性会出错
#             # 但由于 __new__ 会返回已存在的 _instance，这里的 self 应该是有效的，只是其内部状态可能被清空
#             if not LanguageDetectorONNX._instance or not LanguageDetectorONNX._initialized:
#                 raise RuntimeError("LanguageDetectorONNX singleton is not properly initialized or has been released.")
#             # 如果 self.ort_session 是 None 但 _initialized 仍然是 True，说明可能是在 release 过程中
#             # 这里更严格的检查是基于 _initialized
#             if not self.ort_session or not self.tokenizer:
#                 raise RuntimeError("LanguageDetectorONNX resources (session/tokenizer) are missing. Was it released?")
#
#         if not texts:
#             return []
#
#         try:
#             # 1. 预处理输入文本 (批处理)
#             # Tokenizer 会自动处理列表输入，并进行填充 (padding) 以使批次内所有序列长度一致
#             inputs = self.tokenizer(texts, return_tensors="np",
#                                     padding=True,  # 关键：开启填充
#                                     truncation=True,
#                                     max_length=512)  # 根据模型调整 max_length
#
#             # 2. 准备 ONNX Runtime 输入 (确保 int64 类型)
#             ort_inputs = {
#                 'input_ids': inputs['input_ids'].astype(np.int64),
#                 'attention_mask': inputs['attention_mask'].astype(np.int64)
#             }
#             if 'token_type_ids' in inputs:  # 确保 token_type_ids (如果模型需要)
#                 ort_inputs['token_type_ids'] = inputs['token_type_ids'].astype(np.int64)
#
#             # 3. 运行推理 (批处理)
#             # ort_outputs[0] 的形状现在是 [batch_size, num_classes]
#             ort_outputs = self.ort_session.run(None, ort_inputs)
#             batch_logits = ort_outputs[0]  # [batch_size, num_classes]
#
#             results_list = []
#             for i in range(batch_logits.shape[0]):  # 遍历批次中的每个结果
#                 logits = batch_logits[i]  # 当前文本的 logits [num_classes]
#
#                 # 4. 后处理 - 计算概率 (置信度)
#                 probabilities = softmax(logits)  # softmax 在最后一个维度上操作
#                 # 先计算argmax获取最高预测结果
#                 predicted_class_id = np.argmax(probabilities)
#                 predicted_label = self.id2label.get(predicted_class_id, "Unknown")
#                 confidence_score = float(probabilities[predicted_class_id])
#
#                 # 创建临时字典，合并yue和zh
#                 temp_scores = {}
#                 for j, prob in enumerate(probabilities):
#                     lang_code = self.id2label.get(j, f"ID_{j}")
#                     lang_name = Language.get(lang_code).language
#
#                     # 将yue统一标记为zh
#                     if lang_name == 'yue':
#                         lang_name = 'zh'
#
#                     # 如果zh或yue已存在，保留概率较高的值
#                     if lang_name in temp_scores:
#                         temp_scores[lang_name] = max(temp_scores[lang_name], float(prob))
#                     else:
#                         temp_scores[lang_name] = float(prob)
#
#                 # 排序并过滤
#                 sorted_scores = sorted(temp_scores.items(), key=lambda item: item[1], reverse=True)
#                 top_scores = [item for item in sorted_scores if item[1] > 0.0002][:5]
#
#                 # 如果预测标签是yue，也转换为zh
#                 if Language.get(predicted_label).language == 'yue':
#                     predicted_label = 'zh'
#                 else:
#                     predicted_label = Language.get(predicted_label).language
#
#                 results_list.append((i, predicted_label, confidence_score, top_scores))
#
#             # 如果你修改了 predict 方法返回元组，这里也需要对应修改
#             # return [(res['label'], res['confidence'], res['top_3_scores']) for res in results_list]
#             return results_list
#
#         except Exception as e:
#             print(f"Error during batch prediction for {len(texts)} texts. First text: '{texts[0][:50]}...': {e}")
#             import traceback
#             traceback.print_exc()
#             # 对于批处理错误，你可能想返回一个列表，其中每个失败的项都是 None 或错误标记
#             return [None] * len(texts)  # 或者更复杂的错误处理



import threading
import time
import os
import rapidjson as json
from datetime import datetime, timedelta
from ModuleFolders.Base.Base import Base
from ModuleFolders.Infrastructure.TaskConfig.TaskType import TaskType

class QueueTaskItem:
    def __init__(self, task_type, input_path, output_path=None, profile=None, rules_profile=None, 
                 source_lang=None, target_lang=None, project_type=None,
                 platform=None, api_url=None, api_key=None, model=None, 
                 threads=None, retry=None, timeout=None, rounds=None, 
                 pre_lines=None, lines_limit=None, tokens_limit=None, 
                 think_depth=None, thinking_budget=None):
        self.task_type = task_type
        self.input_path = input_path
        self.output_path = output_path
        self.profile = profile
        self.rules_profile = rules_profile
        self.source_lang = source_lang
        self.target_lang = target_lang
        self.project_type = project_type
        
        # 精细化 API 覆盖参数
        self.platform = platform
        self.api_url = api_url
        self.api_key = api_key
        self.model = model
        
        # 性能覆盖参数
        self.threads = threads
        self.retry = retry
        self.timeout = timeout
        self.rounds = rounds
        self.pre_lines = pre_lines
        self.lines_limit = lines_limit
        self.tokens_limit = tokens_limit
        self.think_depth = think_depth
        self.thinking_budget = thinking_budget
        
        self.status = "waiting" # waiting, translating, translated, polishing, completed, error, stopped
        self.locked = False  # 是否被锁定（正在执行中不可修改）

        # 新增：准确的处理状态跟踪
        self.is_processing = False  # 是否真正在处理中
        self.last_activity_time = None  # 最后活动时间（ISO格式字符串）
        self.process_start_time = None  # 处理开始时间

    def to_dict(self):
        d = {k: v for k, v in vars(self).items() if not k.startswith('_')}
        return d

    @classmethod
    def from_dict(cls, data):
        # 兼容旧数据，剔除运行时字段后传入构造函数
        params = data.copy()
        status = params.pop("status", "waiting")
        locked = params.pop("locked", False)  # 移除locked字段，它不属于构造函数参数

        # 新增字段的处理（兼容旧数据）
        is_processing = params.pop("is_processing", False)
        last_activity_time = params.pop("last_activity_time", None)
        process_start_time = params.pop("process_start_time", None)

        item = cls(**params)
        item.status = status
        item.locked = locked
        item.is_processing = is_processing
        item.last_activity_time = last_activity_time
        item.process_start_time = process_start_time
        return item

class QueueManager(Base):
    _instance = None
    _lock = threading.Lock()

    def __new__(cls):
        with cls._lock:
            if cls._instance is None:
                cls._instance = super(QueueManager, cls).__new__(cls)
                cls._instance._initialized = False
            return cls._instance

    def __init__(self):
        if self._initialized: return
        super().__init__()
        # 使用绝对路径确保跨目录一致性
        script_dir = os.path.dirname(os.path.abspath(__file__))
        project_root = os.path.join(script_dir, "..", "..", "..")
        project_root = os.path.normpath(project_root)
        self.default_queue_file = os.path.join(project_root, "Resource", "queue_tasks.json")
        self.queue_file = self.default_queue_file

        # 添加队列操作日志文件
        self.queue_log_file = os.path.join(project_root, "Resource", "queue_operations.log")

        self.tasks = []
        self.is_running = False
        self.current_task_index = -1
        self.load_tasks()
        self._initialized = True

    def _log_queue_operation(self, message):
        """记录队列操作日志到文件，用于跨进程通信"""
        try:
            timestamp = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
            log_entry = f"[{timestamp}] {message}\n"

            # 确保日志目录存在
            os.makedirs(os.path.dirname(self.queue_log_file), exist_ok=True)

            # 追加写入日志文件
            with open(self.queue_log_file, 'a', encoding='utf-8') as f:
                f.write(log_entry)

            # 同时输出到控制台（保留原有行为）
            print(message)

        except Exception as e:
            # 如果日志文件写入失败，至少保证控制台输出
            print(message)
            print(f"[WARNING] Failed to write queue log: {e}")

    def get_queue_log_path(self):
        """获取队列日志文件路径"""
        return self.queue_log_file

    def get_recent_queue_logs(self, lines=10):
        """获取最近的队列操作日志"""
        try:
            if not os.path.exists(self.queue_log_file):
                return []

            with open(self.queue_log_file, 'r', encoding='utf-8') as f:
                all_lines = f.readlines()

            # 返回最后几行，去掉换行符
            recent_lines = all_lines[-lines:] if len(all_lines) >= lines else all_lines
            return [line.strip() for line in recent_lines if line.strip()]

        except Exception as e:
            self.warning(f"Failed to read queue log: {e}")
            return []

    def clear_queue_logs(self):
        """清空队列操作日志"""
        try:
            if os.path.exists(self.queue_log_file):
                with open(self.queue_log_file, 'w', encoding='utf-8') as f:
                    f.write('')  # 清空文件内容
                return True
        except Exception as e:
            self.warning(f"Failed to clear queue log: {e}")
        return False

    def load_tasks(self, custom_path=None):
        if custom_path:
            self.queue_file = custom_path
        if os.path.exists(self.queue_file):
            try:
                with open(self.queue_file, 'r', encoding='utf-8') as f:
                    data = json.load(f)
                    self.tasks = [QueueTaskItem.from_dict(d) for d in data]
            except Exception as e:
                self.error(f"Failed to load queue tasks: {e}")
                self.tasks = []
        else:
            self.tasks = []

    def save_tasks(self):
        try:
            os.makedirs(os.path.dirname(self.queue_file), exist_ok=True)
            with open(self.queue_file, 'w', encoding='utf-8') as f:
                json.dump([t.to_dict() for t in self.tasks], f, indent=4, ensure_ascii=False)
        except Exception as e:
            self.error(f"Failed to save queue tasks: {e}")

    def add_task(self, task_item):
        self.tasks.append(task_item)
        self.save_tasks()

    def remove_task(self, index):
        if 0 <= index < len(self.tasks):
            task_name = os.path.basename(self.tasks[index].input_path)
            self.tasks.pop(index)
            self.save_tasks()
            self.hot_reload_queue(quiet=True)  # 静默热刷新队列
            self._log_queue_operation(Base.i18n.get('msg_task_removed').format(task_name))
            return True
        return False

    def detect_parameter_changes(self, old_task, new_task):
        """检测任务参数变更并返回变更详情"""
        changes = []

        # 定义需要监控的参数及其对应的I18N键
        params_to_monitor = {
            'platform': 'param_platform',
            'api_url': 'param_api_url',
            'api_key': 'param_api_key',
            'model': 'param_model',
            'threads': 'param_threads',
            'source_lang': 'param_source_lang',
            'target_lang': 'param_target_lang',
            'retry': 'param_retry',
            'timeout': 'param_timeout',
            'rounds': 'param_rounds',
            'pre_lines': 'param_pre_lines',
            'lines_limit': 'param_lines_limit',
            'tokens_limit': 'param_tokens_limit',
            'think_depth': 'param_think_depth',
            'thinking_budget': 'param_thinking_budget',
            'task_type': 'param_task_type',
            'profile': 'param_profile',
            'rules_profile': 'param_rules_profile',
            'project_type': 'param_project_type'
        }

        for param, i18n_key in params_to_monitor.items():
            old_value = getattr(old_task, param, None)
            new_value = getattr(new_task, param, None)

            # 对于API密钥等敏感信息，只显示部分内容
            if param == 'api_key':
                if old_value != new_value:
                    old_display = self._mask_sensitive_value(old_value)
                    new_display = self._mask_sensitive_value(new_value)
                    if old_value is None and new_value is not None:
                        changes.append(Base.i18n.get('param_added').format(Base.i18n.get(i18n_key), new_display))
                    elif old_value is not None and new_value is None:
                        changes.append(Base.i18n.get('param_removed').format(Base.i18n.get(i18n_key)))
                    elif old_value != new_value:
                        changes.append(Base.i18n.get('param_changed').format(Base.i18n.get(i18n_key), old_display, new_display))
            else:
                # 普通参数的处理
                if old_value != new_value:
                    if old_value is None and new_value is not None:
                        changes.append(Base.i18n.get('param_added').format(Base.i18n.get(i18n_key), new_value))
                    elif old_value is not None and new_value is None:
                        changes.append(Base.i18n.get('param_removed').format(Base.i18n.get(i18n_key)))
                    elif old_value != new_value:
                        changes.append(Base.i18n.get('param_changed').format(Base.i18n.get(i18n_key), old_value, new_value))

        return changes

    def _mask_sensitive_value(self, value):
        """遮掩敏感信息"""
        if value is None:
            return None
        if len(str(value)) <= 8:
            return "****"
        else:
            return str(value)[:4] + "****" + str(value)[-4:]

    def update_task(self, index, task_item):
        if 0 <= index < len(self.tasks):
            try:
                old_task = self.tasks[index]
                task_name = os.path.basename(old_task.input_path)

                # 检测参数变更
                changes = self.detect_parameter_changes(old_task, task_item)

                # 更新任务
                self.tasks[index] = task_item
                self.save_tasks()
                self.hot_reload_queue(quiet=True)

                # 打印详细的变更日志
                if changes:
                    self._log_queue_operation(Base.i18n.get('msg_task_updated').format(task_name))
                    for change in changes:
                        self._log_queue_operation(change)
                else:
                    # 如果没有参数变更，只是简单的更新
                    self._log_queue_operation(f"[INFO] {Base.i18n.get('msg_task_updated').format(task_name)} {Base.i18n.get('msg_no_config_changes')}")

            except Exception as e:
                self._log_queue_operation(f"[ERROR] Failed to update task: {e}")
                return False

            return True
        return False

    def clear_tasks(self):
        self.tasks = []
        self.save_tasks()
        return True

    def lock_task(self, index):
        """锁定任务（正在执行中）"""
        if 0 <= index < len(self.tasks):
            self.tasks[index].locked = True
            self.save_tasks()
            return True
        return False

    def unlock_task(self, index):
        """解锁任务"""
        if 0 <= index < len(self.tasks):
            self.tasks[index].locked = False
            self.save_tasks()
            return True
        return False

    def can_modify_task(self, index):
        """检查任务是否可以被修改（未锁定）"""
        if 0 <= index < len(self.tasks):
            return not self.tasks[index].locked
        return False

    # ================ 智能处理状态管理 ================

    def update_task_activity(self, index):
        """更新任务活动时间（心跳机制）"""
        if 0 <= index < len(self.tasks):
            self.tasks[index].last_activity_time = datetime.now().isoformat()
            self.save_tasks()
            return True
        return False

    def start_task_processing(self, index):
        """开始处理任务 - 设置处理状态和时间戳"""
        if 0 <= index < len(self.tasks):
            task = self.tasks[index]
            now = datetime.now().isoformat()
            task.is_processing = True
            task.process_start_time = now
            task.last_activity_time = now
            task.locked = True
            self.save_tasks()
            return True
        return False

    def stop_task_processing(self, index):
        """停止处理任务 - 清除处理状态"""
        if 0 <= index < len(self.tasks):
            task = self.tasks[index]
            task.is_processing = False
            task.process_start_time = None
            task.last_activity_time = None
            task.locked = False
            self.save_tasks()
            return True
        return False

    def is_task_actually_processing(self, index, timeout_minutes=5):
        """
        检查任务是否真正在处理中

        Args:
            index: 任务索引
            timeout_minutes: 超时时间（分钟），超过此时间没有活动则认为不在处理中

        Returns:
            bool: 是否真正在处理中
        """
        if not (0 <= index < len(self.tasks)):
            return False

        task = self.tasks[index]

        # 如果明确标记为未处理，直接返回False
        if not task.is_processing:
            return False

        # 检查最后活动时间
        if not task.last_activity_time:
            return False

        try:
            last_activity = datetime.fromisoformat(task.last_activity_time)
            now = datetime.now()
            inactive_time = now - last_activity

            # 如果超过超时时间没有活动，认为不在处理中
            if inactive_time > timedelta(minutes=timeout_minutes):
                self.warning(f"Task {index+1} has been inactive for {inactive_time.total_seconds():.1f} seconds, marking as not processing")
                return False

            return True

        except (ValueError, TypeError) as e:
            self.warning(f"Invalid activity time format for task {index+1}: {e}")
            return False

    def cleanup_stale_locks(self, timeout_minutes=5):
        """
        清理过期的锁定状态

        Args:
            timeout_minutes: 超时时间（分钟）

        Returns:
            int: 清理的任务数量
        """
        cleaned_count = 0

        for i, task in enumerate(self.tasks):
            if task.locked and not self.is_task_actually_processing(i, timeout_minutes):
                self.info(f"Cleaning stale lock for task {i+1}: {task.input_path}")
                task.locked = False
                task.is_processing = False
                task.process_start_time = None
                task.last_activity_time = None

                # 重置状态到合适的值
                if task.status in ["translating", "polishing"]:
                    task.status = "waiting"

                cleaned_count += 1

        if cleaned_count > 0:
            self.save_tasks()
            self.info(f"Cleaned {cleaned_count} stale task locks")

        return cleaned_count

    def get_task_processing_status(self, index):
        """
        获取任务的详细处理状态

        Returns:
            dict: 包含处理状态信息的字典
        """
        if not (0 <= index < len(self.tasks)):
            return None

        task = self.tasks[index]
        is_actually_processing = self.is_task_actually_processing(index)

        status_info = {
            "locked": task.locked,
            "is_processing": task.is_processing,
            "is_actually_processing": is_actually_processing,
            "process_start_time": task.process_start_time,
            "last_activity_time": task.last_activity_time,
            "status": task.status
        }

        return status_info

    def move_task_up(self, index):
        """将指定索引的任务向上移动一位"""
        if (1 <= index < len(self.tasks) and
            self.can_modify_task(index) and self.can_modify_task(index - 1)):
            task_name = os.path.basename(self.tasks[index].input_path)
            self.tasks[index], self.tasks[index - 1] = self.tasks[index - 1], self.tasks[index]
            self.save_tasks()
            self.hot_reload_queue(quiet=True)  # 静默热刷新队列
            self._log_queue_operation(Base.i18n.get('msg_task_moved_up').format(task_name, index+1, index))
            return True
        return False

    def move_task_down(self, index):
        """将指定索引的任务向下移动一位"""
        if (0 <= index < len(self.tasks) - 1 and
            self.can_modify_task(index) and self.can_modify_task(index + 1)):
            task_name = os.path.basename(self.tasks[index].input_path)
            self.tasks[index], self.tasks[index + 1] = self.tasks[index + 1], self.tasks[index]
            self.save_tasks()
            self.hot_reload_queue(quiet=True)  # 静默热刷新队列
            self._log_queue_operation(Base.i18n.get('msg_task_moved_down').format(task_name, index+1, index+2))
            return True
        return False

    def move_task(self, from_index, to_index):
        """将任务从from_index移动到to_index位置"""
        if (0 <= from_index < len(self.tasks) and
            0 <= to_index < len(self.tasks) and
            from_index != to_index and
            self.can_modify_task(from_index)):

            # 检查移动路径上是否有锁定的任务
            start, end = min(from_index, to_index), max(from_index, to_index)
            for i in range(start, end + 1):
                if i != from_index and not self.can_modify_task(i):
                    return False

            # 移除任务
            task = self.tasks.pop(from_index)
            task_name = os.path.basename(task.input_path)
            # 插入到新位置
            self.tasks.insert(to_index, task)
            self.save_tasks()
            self.hot_reload_queue(quiet=True)  # 静默热刷新队列
            self._log_queue_operation(Base.i18n.get('msg_task_moved').format(task_name, from_index+1, to_index+1))
            return True
        return False

    def reorder_tasks(self, new_order):
        """根据新的索引顺序重新排列任务

        Args:
            new_order: 新的索引顺序列表，如 [2, 0, 1] 表示原来的第2个任务移到第0位
        """
        if (len(new_order) == len(self.tasks) and
            set(new_order) == set(range(len(self.tasks)))):

            # 重新排序任务
            self.tasks = [self.tasks[i] for i in new_order]
            self.save_tasks()
            return True
        return False

    def hot_reload_queue(self, quiet=False):
        """热重载队列：在不影响锁定任务的情况下重新加载队列

        Args:
            quiet (bool): 如果为True，不打印成功日志。用于操作后的静默刷新。
        """
        if not os.path.exists(self.queue_file):
            return False

        try:
            # 保存当前锁定状态
            locked_states = {}
            for i, task in enumerate(self.tasks):
                if task.locked:
                    locked_states[i] = {
                        'task_id': f"{task.task_type}_{task.input_path}",
                        'status': task.status
                    }

            # 重新加载任务
            with open(self.queue_file, 'r', encoding='utf-8') as f:
                data = json.load(f)
                new_tasks = [QueueTaskItem.from_dict(d) for d in data]

            # 恢复锁定状态（通过任务特征匹配）
            for i, new_task in enumerate(new_tasks):
                task_id = f"{new_task.task_type}_{new_task.input_path}"
                for old_index, locked_info in locked_states.items():
                    if locked_info['task_id'] == task_id:
                        new_task.locked = True
                        new_task.status = locked_info['status']
                        break

            self.tasks = new_tasks

            # 只有在非静默模式下才打印成功日志
            if not quiet:
                self.info("Queue hot reloaded successfully.")
            return True

        except Exception as e:
            self.error(f"Failed to hot reload queue: {e}")
            return False

    def get_next_unlocked_task(self, start_index=0):
        """获取下一个未锁定的待执行任务"""
        for i in range(start_index, len(self.tasks)):
            task = self.tasks[i]
            if not task.locked and task.status in ["waiting", "translated"]:
                return i, task
        return None, None

    def mark_task_executing(self, index):
        """标记任务为执行中并锁定 - 使用智能处理状态管理"""
        if 0 <= index < len(self.tasks):
            task = self.tasks[index]

            # 使用新的智能处理状态管理
            self.start_task_processing(index)

            # 设置合适的状态
            if task.status == "waiting":
                task.status = "translating"
            elif task.status == "translated":
                task.status = "polishing"

            self.current_task_index = index
            self.save_tasks()
            return True
        return False

    def mark_task_completed(self, index, final_status="completed"):
        """标记任务完成并解锁 - 使用智能处理状态管理"""
        if 0 <= index < len(self.tasks):
            task = self.tasks[index]

            # 使用新的智能处理状态管理
            self.stop_task_processing(index)

            task.status = final_status
            self.save_tasks()
            return True
        return False

    def find_task_by_file_path(self, file_path):
        """根据文件路径查找任务"""
        if not file_path:
            return None

        file_path = os.path.normpath(file_path)
        for i, task in enumerate(self.tasks):
            task_input_path = os.path.normpath(task.input_path)
            if task_input_path == file_path:
                return i, task
        return None, None

    def skip_task_to_end(self, file_path):
        """跳过任务并移动到队列末尾"""
        try:
            task_index, task = self.find_task_by_file_path(file_path)
            if task_index is None:
                return False, "Task not found in queue"

            if not task.locked:
                return False, "Task is not currently locked"

            # 解锁任务并重置状态
            task.locked = False
            task.status = "waiting"

            # 移动到队列末尾
            moved_task = self.tasks.pop(task_index)
            self.tasks.append(moved_task)

            # 保存队列
            self.save_tasks()

            file_name = os.path.basename(file_path)
            self.info(f"Task [{file_name}] skipped and moved to end of queue")
            return True, f"Task moved to position {len(self.tasks)}"

        except Exception as e:
            self.error(f"Failed to skip task: {e}")
            return False, str(e)

    def start_queue(self, cli_menu):
        if self.is_running: return
        self.is_running = True
        threading.Thread(target=self._process_queue, args=(cli_menu,), daemon=True).start()

    def _process_queue(self, cli_menu):
        self.info("Starting task queue processing with full API overrides...")

        # Phase 1: Translation
        while True:
            if Base.work_status == Base.STATUS.STOPING: break

            # 热重载队列
            self.hot_reload_queue(quiet=True)

            # 清理过期的锁定状态
            self.cleanup_stale_locks()

            # 查找下一个需要翻译的任务
            index, task = self.get_next_unlocked_task()
            if index is None:
                break  # 没有更多翻译任务

            if task.task_type not in [TaskType.TRANSLATION, TaskType.TRANSLATE_AND_POLISH]:
                # 标记为完成并继续
                self.mark_task_completed(index, "completed")
                continue

            # 标记任务为执行中
            self.mark_task_executing(index)

            try:
                self._run_single_step(cli_menu, task, TaskType.TRANSLATION)

                # 完成后标记状态
                if task.task_type == TaskType.TRANSLATE_AND_POLISH:
                    self.mark_task_completed(index, "translated")
                else:
                    self.mark_task_completed(index, "completed")
            except Exception as e:
                self.error(f"Task {index+1} failed: {e}")
                self.mark_task_completed(index, "error")

        # Phase 2: Polishing
        if Base.work_status != Base.STATUS.STOPING:
            while True:
                if Base.work_status == Base.STATUS.STOPING: break

                # 热重载队列
                self.hot_reload_queue()

                # 清理过期的锁定状态
                self.cleanup_stale_locks()

                # 查找下一个需要润色的任务
                found_task = False
                for i, task in enumerate(self.tasks):
                    if (not task.locked and
                        task.status == "translated" and
                        task.task_type in [TaskType.POLISH, TaskType.TRANSLATE_AND_POLISH]):

                        found_task = True
                        self.mark_task_executing(i)

                        try:
                            self._run_single_step(cli_menu, task, TaskType.POLISH, resume=True)
                            self.mark_task_completed(i, "completed")
                        except Exception as e:
                            self.error(f"Polish task {i+1} failed: {e}")
                            self.mark_task_completed(i, "error")
                        break

                if not found_task:
                    break  # 没有更多润色任务

        self.is_running = False
        self.info("Task queue processing finished.")

    def _run_single_step(self, cli_menu, task, step_type, resume=False):
        original_active_profile = cli_menu.active_profile_name
        original_rules_profile = cli_menu.active_rules_profile_name
        
        try:
            # 1. Apply Profile Base
            if task.profile: cli_menu.active_profile_name = task.profile
            if task.rules_profile: cli_menu.active_rules_profile_name = task.rules_profile
            cli_menu.load_config()

            # 2. Apply Fine-grained Overrides
            cfg = cli_menu.config
            if task.source_lang: cfg["source_language"] = task.source_lang
            if task.target_lang: cfg["target_language"] = task.target_lang
            if task.project_type: cfg["translation_project"] = task.project_type
            if task.output_path: cfg["label_output_path"] = task.output_path
            
            # --- API Overrides ---
            if task.platform: cfg["target_platform"] = task.platform
            if task.api_url: cfg["base_url"] = task.api_url
            if task.api_key: 
                cfg["api_key"] = task.api_key
                # 同步到具体平台字典中
                tp = cfg.get("target_platform")
                if tp and tp in cfg.get("platforms", {}):
                    cfg["platforms"][tp]["api_key"] = task.api_key
                    
            if task.model: cfg["model"] = task.model
            
            # --- Performance Overrides ---
            if task.threads is not None: cfg["user_thread_counts"] = task.threads
            if task.retry is not None: cfg["retry_count"] = task.retry
            if task.timeout is not None: cfg["request_timeout"] = task.timeout
            if task.rounds is not None: cfg["round_limit"] = task.rounds
            if task.pre_lines is not None: cfg["pre_line_counts"] = task.pre_lines
            
            if task.lines_limit is not None:
                cfg["tokens_limit_switch"] = False
                cfg["lines_limit"] = task.lines_limit
            if task.tokens_limit is not None:
                cfg["tokens_limit_switch"] = True
                cfg["tokens_limit"] = task.tokens_limit
                
            if task.think_depth is not None: cfg["think_depth"] = task.think_depth
            if task.thinking_budget is not None: cfg["thinking_budget"] = task.thinking_budget

            # 3. Execute
            # 更新活动时间（心跳）
            if self.current_task_index >= 0:
                self.update_task_activity(self.current_task_index)

            cli_menu.run_task(step_type, target_path=task.input_path, continue_status=resume, non_interactive=True, from_queue=True)
            
            if Base.work_status != Base.STATUS.STOPING:
                if step_type == TaskType.TRANSLATION and task.task_type == TaskType.TRANSLATE_AND_POLISH:
                    task.status = "translated"
                else:
                    task.status = "completed"
        except Exception as e:
            self.error(f"Task Error: {e}")
            task.status = "error"
        finally:
            self.save_tasks()
            cli_menu.active_profile_name = original_active_profile
            cli_menu.active_rules_profile_name = original_rules_profile
            cli_menu.load_config()


import json
from pathlib import Path

from ModuleFolders.Infrastructure.Cache.CacheFile import CacheFile
from ModuleFolders.Infrastructure.Cache.CacheItem import CacheItem
from ModuleFolders.Infrastructure.Cache.CacheProject import ProjectType
from ModuleFolders.Domain.FileReader.BaseReader import (
    BaseSourceReader,
    InputConfig,
    PreReadMetadata
)


class VntReader(BaseSourceReader):
    def __init__(self, input_config: InputConfig):
        super().__init__(input_config)

    @classmethod
    def get_project_type(cls):
        return ProjectType.VNT

    @property
    def support_file(self):
        return "json"

    def on_read_source(self, file_path: Path, pre_read_metadata: PreReadMetadata) -> CacheFile:
        json_data = json.loads(file_path.read_text(encoding="utf-8"))
        items = []
        for entry in json_data:
            source_text = entry["message"]
            names = entry.get("names", [])  # 默认获取空列表
            name = entry.get("name", "")    # 默认获取空字符串

            extra = {}
            if names:
                # 处理names列表，拼接每个名字
                name_tags = ''.join([f'[{n}]' for n in names])
                source_text = f"{name_tags}{source_text}"
                extra["names"] = names
            elif name:
                # 处理单个name字段
                source_text = self.combine_srt(name, source_text)
                extra["name"] = name
            # 无name或names字段时不处理
            item = CacheItem(source_text=source_text, extra=extra)
            items.append(item)
        return CacheFile(items=items)

    def combine_srt(self, name, text):
        return f"[{name}]{text}"

    def can_read_by_content(self, file_path: Path) -> bool:
        # [{"mesage": "text1"}, {"message": "text2"}]
        # 即使不是对应编码也不影响英文的key
        content = json.loads(file_path.read_text(encoding="utf-8", errors='ignore'))
        if not isinstance(content, list):
            return False
        return all(isinstance(line, dict) and 'message' in line for line in content)





# 检查接口是否拒绝翻译，而返回一段话
def contains_special_chars(s: str) -> bool:
    special_chars = ['<', '>', '/']
    return any(char in s for char in special_chars)

# 检查数字序号是否正确
def check_dict_order(source_text_dict,input_dict):
    """
    检查输入的字典，字典的key是从零开始增加的字符数字，值是文本。
    顺序检查每个值的开头是否是以数字序号+英文句点开头，并且是从1开始增加的数字序号，
    全部检查通过返回真，反之返回假。

    Args:
        input_dict (dict): 输入的字典，key为字符数字，value为文本。

    Returns:
        bool: 检查全部通过返回True，否则返回False。
    """
    if (len(source_text_dict) == 1) and (len(input_dict) == 1):
        return True  # 一行就不检查了


    expected_num = 1  # 期望的起始序号
    keys = sorted(input_dict.keys(), key=int)  # 获取排序后的key，确保按数字顺序检查

    for key in keys:
        value = input_dict[key]
        prefix = str(expected_num) + "."
        if not value.startswith(prefix):
            return False  # 值没有以期望的序号开头
        expected_num += 1  # 序号递增

    return True  # 所有检查都通过

# 检查回复内容的文本行数
def check_text_line_count(source_dict, response_dict):
    return (
        len(source_dict) > 0 and len(response_dict) > 0 # 数据不为空
        and len(source_dict) == len(response_dict) # 原文与译文行数一致
        and all(str(key) in response_dict for key in range(len(source_dict))) # 译文的 Key 的值为从 0 开始的连续数值字符
    )

# 检查翻译内容是否有空值
def check_empty_response(response_dict):
    for value in response_dict.values():
        #检查value是不是None，因为AI回回复null，但是json.loads()会把null转化为None
        if value is None:
            return False

        # 检查value是不是空字符串，因为AI回回复空字符串，但是json.loads()会把空字符串转化为""
        if value == "":
            return False

    return True




from pathlib import Path

from ModuleFolders.Infrastructure.Cache.CacheFile import CacheFile
from ModuleFolders.Infrastructure.Cache.CacheProject import ProjectType
from ModuleFolders.Domain.FileReader.BaseReader import (
    BaseSourceReader,
    InputConfig,
    PreReadMetadata
)
from ModuleFolders.Domain.FileReader.TxtReader import TxtReader


class MdReader(BaseSourceReader):
    def __init__(self, input_config: InputConfig):
        super().__init__(input_config)
        self.txt_reader = TxtReader(input_config, None)  # 简单复用TxtReader，如有另外实现请删除

    def __enter__(self):
        self.txt_reader.__enter__()
        return super().__enter__()

    def __exit__(self, exc_type, exc, exc_tb):
        self.txt_reader.__exit__(exc_type, exc, exc_tb)
        return super().__exit__(exc_type, exc, exc_tb)

    @classmethod
    def get_project_type(cls):
        return ProjectType.MD

    @property
    def support_file(self):
        return "md"

    def read_source_file(self, file_path: Path) -> CacheFile:
        cache_file = self.txt_reader.read_source_file(file_path)
        for item in cache_file.items:
            item.set_extra("original_line", item.source_text)
        return cache_file

    def on_read_source(self, file_path: Path, pre_read_metadata: PreReadMetadata) -> CacheFile:
        # 重载抽象方法，实际不需要使用
        raise NotImplementedError



"""
异步任务执行器 - 基于 asyncio + aiohttp 的高并发翻译引擎

核心特性：
- 使用 asyncio 事件循环管理所有并发任务
- 复用 aiohttp 连接池，减少 TCP 握手开销
- 集成 StreamBuffer 实现零碎片数据流合并
- 支持动态并发控制和优雅停止
"""

import asyncio
import time
from typing import List, Dict, Any, Optional, Callable
from dataclasses import dataclass, field

from ModuleFolders.Base.Base import Base
from ModuleFolders.Infrastructure.Cache.CacheItem import CacheItem, TranslationStatus
from ModuleFolders.Infrastructure.Cache.StreamBuffer import StreamBuffer
from ModuleFolders.Infrastructure.LLMRequester.AsyncOpenaiRequester import AsyncOpenaiRequester


@dataclass
class AsyncTaskResult:
    """异步任务结果"""
    task_id: str
    success: bool
    row_count: int = 0
    prompt_tokens: int = 0
    completion_tokens: int = 0
    error_message: str = ""
    translated_items: Dict[int, str] = field(default_factory=dict)


class AsyncTaskExecutor(Base):
    """
    异步任务执行器

    使用方式：
    1. 创建实例并配置
    2. 调用 execute_tasks() 执行所有任务
    3. 通过回调获取进度更新
    """

    def __init__(
        self,
        max_concurrency: int = 100,
        on_task_complete: Optional[Callable[[AsyncTaskResult], None]] = None,
        on_progress_update: Optional[Callable[[int, int], None]] = None,
    ):
        super().__init__()
        self._max_concurrency = max_concurrency
        self._on_task_complete = on_task_complete
        self._on_progress_update = on_progress_update

        # 并发控制
        self._semaphore: Optional[asyncio.Semaphore] = None
        self._running = False
        self._stop_requested = False

        # 统计数据
        self._total_tasks = 0
        self._completed_tasks = 0
        self._failed_tasks = 0

        # 流式缓冲区
        self._stream_buffer = StreamBuffer()

    @property
    def max_concurrency(self) -> int:
        return self._max_concurrency

    @max_concurrency.setter
    def max_concurrency(self, value: int) -> None:
        """动态调整并发数"""
        self._max_concurrency = max(1, min(value, 500))
        # 注意：已创建的 semaphore 不会自动更新
        # 新的限制将在下一批任务中生效

    def request_stop(self) -> None:
        """请求停止执行"""
        self._stop_requested = True

    async def _execute_single_task(
        self,
        task_id: str,
        items: List[CacheItem],
        messages: list,
        system_prompt: str,
        platform_config: dict,
    ) -> AsyncTaskResult:
        """执行单个翻译任务"""
        result = AsyncTaskResult(task_id=task_id, success=False)

        if self._stop_requested:
            result.error_message = "Task cancelled"
            return result

        try:
            # 获取信号量（控制并发）
            async with self._semaphore:
                if self._stop_requested:
                    result.error_message = "Task cancelled"
                    return result

                # 发起异步请求
                requester = AsyncOpenaiRequester()
                skip, think, content, pt, ct = await requester.request_openai_async(
                    messages, system_prompt, platform_config
                )

                result.prompt_tokens = pt
                result.completion_tokens = ct

                if skip:
                    result.error_message = content
                    return result

                # 简单的结果提取（实际应使用 ResponseExtractor）
                if content:
                    result.success = True
                    result.row_count = len(items)

                    # 将结果写入 items
                    lines = content.strip().split('\n')
                    for i, item in enumerate(items):
                        if i < len(lines):
                            result.translated_items[item.text_index] = lines[i]

                return result

        except Exception as e:
            result.error_message = str(e)
            return result

    async def _task_wrapper(
        self,
        task_id: str,
        items: List[CacheItem],
        messages: list,
        system_prompt: str,
        platform_config: dict,
    ) -> AsyncTaskResult:
        """任务包装器，处理回调和统计"""
        result = await self._execute_single_task(
            task_id, items, messages, system_prompt, platform_config
        )

        # 更新统计
        if result.success:
            self._completed_tasks += 1
        else:
            self._failed_tasks += 1

        # 写入流式缓冲区
        self._stream_buffer.write_chunk(task_id, result, result.success)

        # 触发回调
        if self._on_task_complete:
            try:
                self._on_task_complete(result)
            except Exception as e:
                self.error(f"Task callback error: {e}")

        if self._on_progress_update:
            try:
                total_done = self._completed_tasks + self._failed_tasks
                self._on_progress_update(total_done, self._total_tasks)
            except Exception:
                pass

        return result

    async def execute_tasks_async(
        self,
        tasks: List[Dict[str, Any]],
    ) -> List[AsyncTaskResult]:
        """
        异步执行所有任务

        Args:
            tasks: 任务列表，每个任务包含:
                - task_id: 任务ID
                - items: CacheItem 列表
                - messages: 消息列表
                - system_prompt: 系统提示词
                - platform_config: 平台配置

        Returns:
            所有任务的结果列表
        """
        self._running = True
        self._stop_requested = False
        self._total_tasks = len(tasks)
        self._completed_tasks = 0
        self._failed_tasks = 0

        # 创建信号量
        self._semaphore = asyncio.Semaphore(self._max_concurrency)

        # 预分配流式缓冲区
        task_ids = [t["task_id"] for t in tasks]
        self._stream_buffer.prepare(task_ids)

        # 创建所有协程任务
        coroutines = [
            self._task_wrapper(
                task["task_id"],
                task["items"],
                task["messages"],
                task["system_prompt"],
                task["platform_config"],
            )
            for task in tasks
        ]

        # 并发执行所有任务
        results = await asyncio.gather(*coroutines, return_exceptions=True)

        # 处理异常结果
        final_results = []
        for i, result in enumerate(results):
            if isinstance(result, Exception):
                final_results.append(AsyncTaskResult(
                    task_id=tasks[i]["task_id"],
                    success=False,
                    error_message=str(result)
                ))
            else:
                final_results.append(result)

        self._running = False

        # 关闭连接池
        await AsyncOpenaiRequester.close_session()

        return final_results

    def execute_tasks(self, tasks: List[Dict[str, Any]]) -> List[AsyncTaskResult]:
        """
        同步接口：执行所有任务

        这是对 execute_tasks_async 的同步包装，
        方便在非异步代码中调用。
        """
        return asyncio.run(self.execute_tasks_async(tasks))

    def get_statistics(self) -> Dict[str, Any]:
        """获取执行统计"""
        return {
            "total": self._total_tasks,
            "completed": self._completed_tasks,
            "failed": self._failed_tasks,
            "running": self._running,
            "buffer_stats": self._stream_buffer.finalize() if not self._running else None
        }



"""
流式缓冲区模块 - 零碎片数据流合并方案

解决痛点：
- 100个并发回来时，避免多次内存分配和字符串拼接
- 预先申请内存缓冲区，每个请求带 chunk_id 直接写入对应位置
- 最后一个请求结束时，整本书已在内存中拼装完成
"""

import threading
from dataclasses import dataclass, field
from typing import Dict, Any, Optional, Callable
from collections import OrderedDict


@dataclass
class ChunkSlot:
    """单个数据槽位"""
    chunk_id: str
    status: int = 0  # 0=pending, 1=completed, 2=failed
    data: Any = None
    byte_size: int = 0


class StreamBuffer:
    """
    流式缓冲区 - 预分配槽位的零碎片合并器

    使用方式：
    1. 任务开始前调用 prepare() 预分配所有槽位
    2. 每个并发任务完成后调用 write_chunk() 写入对应槽位
    3. 所有任务完成后调用 finalize() 获取完整结果

    特点：
    - 预分配槽位，避免动态扩容
    - 直接写入对应位置，无需后期合并
    - 线程安全，支持高并发写入
    - 支持进度追踪和完成回调
    """

    def __init__(self, on_complete: Optional[Callable] = None):
        self._lock = threading.Lock()
        self._slots: OrderedDict[str, ChunkSlot] = OrderedDict()
        self._total_chunks = 0
        self._completed_chunks = 0
        self._failed_chunks = 0
        self._on_complete = on_complete
        self._is_finalized = False

    def prepare(self, chunk_ids: list[str]) -> None:
        """
        预分配所有槽位

        Args:
            chunk_ids: 所有 chunk 的 ID 列表，按顺序排列
        """
        with self._lock:
            self._slots.clear()
            self._total_chunks = len(chunk_ids)
            self._completed_chunks = 0
            self._failed_chunks = 0
            self._is_finalized = False

            for chunk_id in chunk_ids:
                self._slots[chunk_id] = ChunkSlot(chunk_id=chunk_id)

    def write_chunk(self, chunk_id: str, data: Any, success: bool = True) -> bool:
        """
        写入单个 chunk 的数据到对应槽位

        Args:
            chunk_id: chunk 的唯一标识
            data: 要写入的数据
            success: 是否成功

        Returns:
            bool: 是否所有 chunk 都已完成
        """
        all_done = False

        with self._lock:
            if chunk_id not in self._slots:
                return False

            slot = self._slots[chunk_id]
            slot.data = data
            slot.status = 1 if success else 2

            if success:
                self._completed_chunks += 1
            else:
                self._failed_chunks += 1

            # 检查是否全部完成
            all_done = (self._completed_chunks + self._failed_chunks) >= self._total_chunks

        # 在锁外触发回调，避免死锁
        if all_done and self._on_complete:
            self._on_complete(self)

        return all_done

    def get_progress(self) -> tuple[int, int, int]:
        """
        获取当前进度

        Returns:
            tuple: (已完成数, 失败数, 总数)
        """
        with self._lock:
            return (self._completed_chunks, self._failed_chunks, self._total_chunks)

    def is_all_done(self) -> bool:
        """检查是否所有 chunk 都已处理"""
        with self._lock:
            return (self._completed_chunks + self._failed_chunks) >= self._total_chunks

    def get_completed_data(self) -> list[Any]:
        """
        获取所有已完成的数据（按原始顺序）

        Returns:
            list: 所有成功完成的 chunk 数据
        """
        with self._lock:
            return [
                slot.data
                for slot in self._slots.values()
                if slot.status == 1 and slot.data is not None
            ]

    def get_all_data_ordered(self) -> OrderedDict[str, Any]:
        """
        获取所有数据（按原始顺序，包含失败的）

        Returns:
            OrderedDict: chunk_id -> data 的有序字典
        """
        with self._lock:
            return OrderedDict(
                (slot.chunk_id, slot.data)
                for slot in self._slots.values()
            )

    def finalize(self) -> dict:
        """
        完成缓冲区，返回统计信息

        Returns:
            dict: 包含完成统计的字典
        """
        with self._lock:
            self._is_finalized = True
            return {
                "total": self._total_chunks,
                "completed": self._completed_chunks,
                "failed": self._failed_chunks,
                "success_rate": self._completed_chunks / self._total_chunks if self._total_chunks > 0 else 0
            }

    def reset(self) -> None:
        """重置缓冲区"""
        with self._lock:
            self._slots.clear()
            self._total_chunks = 0
            self._completed_chunks = 0
            self._failed_chunks = 0
            self._is_finalized = False


class IndexedResultBuffer:
    """
    索引结果缓冲区 - 基于文本索引的预分配方案

    专门用于翻译结果的零碎片合并：
    - 预先知道所有 text_index
    - 每个翻译结果直接写入对应索引位置
    - 支持按文件分组的结果收集
    """

    def __init__(self):
        self._lock = threading.Lock()
        # 结构: {file_path: {text_index: translated_text}}
        self._buffers: Dict[str, Dict[int, str]] = {}
        self._pending_counts: Dict[str, int] = {}
        self._completed_counts: Dict[str, int] = {}

    def prepare_file(self, file_path: str, text_indices: list[int]) -> None:
        """
        为单个文件预分配缓冲区

        Args:
            file_path: 文件路径
            text_indices: 该文件中所有待翻译的 text_index 列表
        """
        with self._lock:
            self._buffers[file_path] = {idx: None for idx in text_indices}
            self._pending_counts[file_path] = len(text_indices)
            self._completed_counts[file_path] = 0

    def write_result(self, file_path: str, text_index: int, translated_text: str) -> bool:
        """
        写入单个翻译结果

        Args:
            file_path: 文件路径
            text_index: 文本索引
            translated_text: 翻译结果

        Returns:
            bool: 该文件是否全部完成
        """
        with self._lock:
            if file_path not in self._buffers:
                return False
            if text_index not in self._buffers[file_path]:
                return False

            # 只有首次写入才计数
            if self._buffers[file_path][text_index] is None:
                self._completed_counts[file_path] += 1

            self._buffers[file_path][text_index] = translated_text

            return self._completed_counts[file_path] >= self._pending_counts[file_path]

    def write_batch(self, file_path: str, results: Dict[int, str]) -> bool:
        """
        批量写入翻译结果

        Args:
            file_path: 文件路径
            results: {text_index: translated_text} 字典

        Returns:
            bool: 该文件是否全部完成
        """
        with self._lock:
            if file_path not in self._buffers:
                return False

            for text_index, translated_text in results.items():
                if text_index in self._buffers[file_path]:
                    if self._buffers[file_path][text_index] is None:
                        self._completed_counts[file_path] += 1
                    self._buffers[file_path][text_index] = translated_text

            return self._completed_counts[file_path] >= self._pending_counts[file_path]

    def get_file_progress(self, file_path: str) -> tuple[int, int]:
        """
        获取单个文件的进度

        Returns:
            tuple: (已完成数, 总数)
        """
        with self._lock:
            if file_path not in self._buffers:
                return (0, 0)
            return (
                self._completed_counts.get(file_path, 0),
                self._pending_counts.get(file_path, 0)
            )

    def get_file_results(self, file_path: str) -> Dict[int, str]:
        """
        获取单个文件的所有结果

        Returns:
            dict: {text_index: translated_text}
        """
        with self._lock:
            return dict(self._buffers.get(file_path, {}))

    def is_file_complete(self, file_path: str) -> bool:
        """检查单个文件是否全部完成"""
        with self._lock:
            if file_path not in self._buffers:
                return False
            return self._completed_counts[file_path] >= self._pending_counts[file_path]

    def get_total_progress(self) -> tuple[int, int]:
        """
        获取总体进度

        Returns:
            tuple: (已完成总数, 总数)
        """
        with self._lock:
            total_completed = sum(self._completed_counts.values())
            total_pending = sum(self._pending_counts.values())
            return (total_completed, total_pending)

    def clear(self) -> None:
        """清空所有缓冲区"""
        with self._lock:
            self._buffers.clear()
            self._pending_counts.clear()
            self._completed_counts.clear()



import re
from typing import Dict, List

import rich
from rich.markup import escape


# 回复解析器
class ResponseExtractor:

    # 多行文本数字匹配格式
    # 目前有两个匹配组，第一个为标准数字序号部分，第二个为可能出现的多余引号组（常见于deepseek-v3）
    multiline_number_prefix = r'(\d+\.\d+\.)(")?[,，\s]?(?(2)"|)?'
    # 多行文本段结束后缀
    multiline_quote_suffix = r'(?:\n["“”][,，]|["“”]\n[,，]|["“”]?[,，]?)$'

    # 判断多行文本开始的正则
    multiline_start_reg = re.compile(rf'^\s*["“”]\s*{multiline_number_prefix}')
    # 提取多行文本边界的正则
    boundary_pattern_reg = re.compile(f'["“”][^"“”]*?{multiline_number_prefix}')
    # 提取规范数字序号与正文的正则
    extract_num_text_reg = re.compile(f'["“”][^"“”]*?{multiline_number_prefix}(.*?){multiline_quote_suffix}')

    def __init__(self):
        pass

    # 处理并正则提取翻译内容
    def text_extraction(self, source_text_dict, response_content):

        try:
            # 提取译文结果
            translation_result= ResponseExtractor.extract_translation(self,source_text_dict,response_content)

            return translation_result
        except Exception as e:
            print(f"\033[1;33mWarning:\033[0m 回复内容无法正常提取，请反馈\n错误信息: {str(e)}")
            return {},{},{}

    # 提取翻译结果内容
    def extract_translation(self,source_text_dict,html_string):

        # 提取翻译文本
        text_dict = ResponseExtractor.label_text_extraction(self,source_text_dict,html_string)

        if not text_dict:
            return {}  # 如果没有找到标签内容，返回空 JSON

        # 计算原文行数
        newlines_in_dict = ResponseExtractor.count_newlines_in_dict_values(self,source_text_dict)

        # 合并调整翻译文本
        translation_result= ResponseExtractor.generate_text_by_newlines(self,newlines_in_dict,text_dict)

        return translation_result

    # 辅助函数，正则提取标签文本内容
    def label_text_extraction(self,source_text_dict, html_string):

        # 只提取最后一个 textarea 标签的内容
        textarea_contents = re.findall(r'<textarea.*?>(.*?)</textarea>', html_string, re.DOTALL)
        if not textarea_contents:
            return {}  # 如果没有找到 textarea 标签，返回空字典
        last_content = textarea_contents[-1]

        # 提取文本存储到字典中
        output_dict = ResponseExtractor.extract_text_to_dict(self,last_content)

        # 如果原文是一行，则跳过过滤，主要是本地模型兼容
        if len(source_text_dict) == 1 :
             return output_dict 

        # 从第一个以数字序号开头的行开始，保留之后的所有行(主要是有些AI会在译文内容前面加点说明)
        filtered_dict = {}
        found = False
        # 按行号顺序遍历
        for key in sorted(output_dict.keys(), key=lambda k: int(k)):
            value = output_dict[key]
            if not found:
                if re.match(r'^\d+\.', value):  # 匹配以数字和句点开头的行
                    found = True
                else:
                    continue
            if found:
                filtered_dict[key] = value

        return filtered_dict

    # 提取文本为字典
    def extract_text_to_dict(self, input_string: str) -> Dict[str, str]:
        """
        从特定格式的字符串中提取内容并存入字典 。
        （目前因为AI偶尔会合并翻译，回复的列表块最后一个元素内容是这样的:"3.1."或者"3.1.[换行符]文本内容"，所以后期会检查出换行符不一致）
        Args:
            input_string: 输入的字符串。

        Returns:
            一个字典，键是'0', '1', '2'...，值是提取到的文本行。
        """
        # 1. 初步分割: 按主序号分割成块
        blocks = re.split(r'\n(?=\d+\.)', input_string.strip())

        extracted_items = []

        # 2. 处理每个块
        for block in blocks:
            block = block.strip()
            if not block:
                continue

            # 3. 尝试匹配列表块模式 (N.[ ... ])
            # re.DOTALL 使 '.' 可以匹配换行符
            list_block_match = re.match(r'^\d+\.\s*\[(.*)]$', block, re.DOTALL)
            if list_block_match:
                list_content = list_block_match.group(1).strip()
                # 再次判断是否是列表块内容(通过以特定模式开始作为判断)
                if list_content and ResponseExtractor.multiline_start_reg.match(list_content):
                    items = ResponseExtractor.extract_multiline_content(self, list_content)
                    extracted_items.extend(items)
                else:
                    # 如果方括号内的内容不像带引号列表 (例如 "9.[社团活动后]")
                    # 将整个原始块（包括 N.[...] ）视为一个单独的文本项。
                    extracted_items.append(block)
            else:
                # 4.2 文本块: 不是 N.[...] 格式，直接添加整个块内容
                extracted_items.append(block)

        # 5. 生成最终字典
        result_dict = {str(i): item for i, item in enumerate(extracted_items)}
        return result_dict

    def extract_multiline_content(self, text: str) -> List[str]:
        """
        从文本中提取引号包围的内容，处理嵌套引号。

        Args:
            text: 要处理的文本

        Returns:
            提取出的内容列表
            从捕获组1获取标准序号`n.n.`，然后从捕获组3获取正文，随后组装这两个组（以英文逗号分隔）并返回
        """
        result = []

        # 首先尝试找到所有可能的列表项边界
        # 这个模式匹配：引号 + 可能的空白 + 数字.数字.,
        boundaries = [m.start() for m in ResponseExtractor.boundary_pattern_reg.finditer(text)]

        # 如果找不到边界，返回空列表
        if not boundaries:
            return []

        # 添加文本结束作为最后一个边界
        boundaries.append(len(text))

        # 逐对处理边界
        for i in range(len(boundaries) - 1):
            segment = text[boundaries[i]:boundaries[i + 1]].strip()

            # 目前的 `multiline_number_prefix` 中有两个匹配组：
            # 第一个匹配数字序号n.n. 第二个匹配可能出现的多余引号
            # 因此这里的 `extract_num_text_reg` 使用第三个匹配组进行正文获取
            match = ResponseExtractor.extract_num_text_reg.search(segment)

            if match:
                try:
                    # 提取第1组(数字)和第3组(文本)
                    number_part = match.group(1)
                    text_part = match.group(3)

                    # 确保两个组都被成功捕获
                    if number_part is not None and text_part is not None:
                        # 去除`text_part`中可能出现的`number_part`
                        cleaned_text_part = text_part.replace(number_part, '').replace(number_part.rstrip('.'), '')
                        # 组合数字和文本，保留匹配到的 `text_part` 原始文本
                        assembled_content = f"{number_part},{cleaned_text_part}"
                        result.append(assembled_content)
                    else:
                        # 更详细地指明哪个部分为空
                        missing_parts = []
                        if number_part is None:
                            missing_parts.append("数字部分")
                        if text_part is None:
                            missing_parts.append("文本部分")

                        # 输出更详细的警告信息
                        rich.print(
                            f"[[red]WARNING[/]] 多行文本 提取失败：{', '.join(missing_parts)}为空。"
                            f"原始文本：{escape(match.group(0))[:50]}{'...' if len(match.group(0)) > 50 else ''} "
                        )
                except IndexError as e:
                    # 处理组不存在的情况，添加更详细的错误信息
                    rich.print(
                        f"[[red]WARNING[/]] 多行文本 提取索引错误：{str(e)}。无法从匹配中提取预期组。"
                        f"原始文本：{escape(match.group(0))[:50]}{'...' if len(match.group(0)) > 50 else ''} "
                    )
            else:
                # 添加未匹配情况的日志
                if segment and len(segment.strip()) > 0:  # 只记录非空段落
                    rich.print(
                        f"[[red]WARNING[/]] 多行文本 未找到匹配段落：{escape(segment[:50])}{'...' if len(segment) > 50 else ''} "
                    )

        return result

    # 辅助函数，统计原文中的换行符
    def count_newlines_in_dict_values(self,source_text_dict):
        """
        统计字典中每个文本值内的换行符数量，并生成一个包含换行符统计结果的新字典。

        Args:
            source_text_dict: 输入字典，键为字符串，值为文本字符串。

        Returns:
            一个字典，键与输入字典相同，值为对应文本字符串中的换行符数量。
            例如：
            {'0': 0, '1': 1, '2': 0}
        """
        newline_counts = {}  # 初始化一个空字典用于存储换行符数量

        for key, text in source_text_dict.items():
            newline_count = text.count('\n')  # 使用字符串的count()方法统计换行符数量
            if newline_count == 0:
                newline_counts[key] = newline_count  # 将统计结果存入新字典，键保持不变
            else:
                newline_counts[key] = newline_count
        return newline_counts

    # 辅助函数，根据换行符数量生成最终译文字典，与原文字典进行一一对应
    def generate_text_by_newlines(self, newline_counts_dict, translation_text_dict):
        """
        根据换行符统计字典和源文本字典生成新的字典，并处理多余内容。

        Args:
            newline_counts_dict: 换行符统计字典，键为字符串序号，值为换行符数量。
            translation_text_dict: 源文本字典，键为字符串序号，值为文本内容。

        Returns:
            一个新的字典，键与 newline_counts_dict 相同，值为根据换行符数量从 translation_text_dict
            中提取并拼接的文本内容。如果 translation_text_dict 内容不足，对应值为空字符串。
            如果 translation_text_dict 有剩余内容，将添加新键值对，键为最大原键加1，值为剩余内容按换行拼接。
        """
        result_dict = {}
        translation_keys_sorted = sorted(translation_text_dict.keys(), key=int)
        translation_index = 0

        # 处理每个预定键的行数需求
        for key, newline_count in newline_counts_dict.items():
            extracted_lines = []
            num_lines_to_extract = newline_count + 1 if newline_count > 0 else 1

            for _ in range(num_lines_to_extract):
                if translation_index < len(translation_keys_sorted):
                    current_key = translation_keys_sorted[translation_index]
                    extracted_lines.append(translation_text_dict[current_key])
                    translation_index += 1
                else:
                    break

            if extracted_lines:
                result_dict[key] = '\n'.join(extracted_lines) if newline_count > 0 else extracted_lines[0]
            else:
                result_dict[key] = ''

        # 添加剩余内容为新键值对
        if translation_index < len(translation_keys_sorted):
            if newline_counts_dict:
                max_key = max(map(int, newline_counts_dict.keys()))
            else:
                max_key = 0
            new_key = str(max_key + 1)
            remaining_texts = [translation_text_dict[k] for k in translation_keys_sorted[translation_index:]]
            result_dict[new_key] = '\n'.join(remaining_texts)

        return result_dict


    # 去除数字序号及括号
    def remove_numbered_prefix(self, translation_text_dict):
        """
        去除翻译文本中的数字序号前缀。
        
        处理两个步骤：
        1. 去除各种变形序号（包括特殊前缀字符 + 数字序号 + 特殊后缀标点）
        2. 循环去除开头剩余的简单数字序号
        
        Args:
            translation_text_dict: 翻译文本字典
            
        Returns:
            处理后的文本字典
        """
        output_dict = {}
        for key, value in translation_text_dict.items():

            if not isinstance(value, str):
                output_dict[key] = value
                continue
                
            translation_lines = value.split('\n')
            cleaned_lines = []
            
            for i, line in enumerate(translation_lines):
                temp_line = line
                
                # 第一步：去除各种变形序号
                # 匹配模式：[可选空白][可选特殊前缀][数字序号][可选标点后缀][可选空白]
                # - 特殊前缀：「『【……□ 等引号、省略号、方框等字符
                # - 数字序号：支持 1. 或 1.2. 或 1.2.3. 等多级序号
                # - 标点后缀：, ， 、等中英文标点
                temp_line = re.sub(
                    r'^\s*[「『【（\(……□\s]*\d+(\.\d+)*\.[,，、]?\s*',
                    '',
                    temp_line
                )
                
                # 第二步：循环去除开头剩余的 数字. 格式
                # 处理可能存在的嵌套或多余的数字序号
                max_iterations = 2  # 设置最大迭代次数，防止意外无限循环
                for iteration in range(max_iterations):
                    new_line = re.sub(r'^\s*\d+\.\s*', '', temp_line)
                    # 如果没有变化，说明已经清理完毕
                    if new_line == temp_line:
                        break
                    temp_line = new_line
                
                cleaned_lines.append(temp_line)

            processed_text = '\n'.join(cleaned_lines)

            # 移除尾部的 "/n] 或 /n] (及其前面的空格)
            final_text = re.sub(r'\s*"?\n]$', '', processed_text)
            output_dict[key] = final_text
            
        return output_dict